@@ -37,8 +37,11 @@
 #include "katemailfilesdialog.h"
 #include "katemainwindowiface.h"
 #include "kateexternaltools.h"
+#include "katesavemodifieddialog.h"
+#include "katemwmodonhddialog.h"
+
+#include <kmdi/tabwidget.h>
 
-#include <kmdichildview.h>
 #include <dcopclient.h>
 #include <kinstance.h>
 #include <kaboutdata.h>
@@ -73,7 +76,7 @@
 #include <kmultitabbar.h>
 #include <ktip.h>
 #include <kmenubar.h>
-
+#include <kstringhandler.h>
 #include <qlayout.h>
 #include <qptrvector.h>
 
@@ -82,26 +85,22 @@
 //END
 
 uint KateMainWindow::uniqueID = 1;
-KMdi::MdiMode KateMainWindow::defaultMode=KMdi::UndefinedMode;
 
-KateMainWindow::KateMainWindow(KateDocManager *_m_docManager, KatePluginManager *_m_pluginManager,
-	KateProjectManager *projectMan, KMdi::MdiMode guiMode) :
-	KMdiMainFrm (0,(QString("__KateMainWindow#%1").arg(uniqueID)).latin1(),guiMode)
+KateMainWindow::KateMainWindow () :
+    KMDI::MainWindow (0,(QString("__KateMainWindow#%1").arg(uniqueID)).latin1())
 {
-  setToolviewStyle(KMultiTabBar::KDEV3ICON);
+  setToolViewStyle(KMultiTabBar::KDEV3ICON);
+  // make the dockwidgets keep their size if possible
+  manager()->setSplitterKeepSize(true);
   // first the very important id
   myID = uniqueID;
   uniqueID++;
 
   // init some vars
-  m_docManager =  _m_docManager;
-  m_pluginManager =_m_pluginManager;
-  m_projectManager = projectMan;
-
   m_project = 0;
   m_projectNumber = 0;
 
-  activeView = 0;
+  m_modignore = false;
 
   console = 0;
   greptool = 0;
@@ -121,10 +120,11 @@ KateMainWindow::KateMainWindow(KateDocManager *_m_docManager, KatePluginManager
     resize (kMin (s.width(), desk.width()), kMin(s.height(), desk.height()));
   }
 
+  // apply settings
+  applyMainWindowSettings(kapp->config(), "Kate Main Window");
+
   m_mainWindow = new Kate::MainWindow (this);
   m_toolViewManager = new Kate::ToolViewManager (this);
-  setStandardMDIMenuEnabled(false);
-  setManagedDockPositionModeEnabled(true);
 
   m_dcop = new KateMainWindowDCOPIface (this);
 
@@ -133,25 +133,24 @@ KateMainWindow::KateMainWindow(KateDocManager *_m_docManager, KatePluginManager
 
   // setup the actions
   setupActions();
-  projectlist->setupActions();
 
   setStandardToolBarMenuEnabled( true );
   setXMLFile( "kateui.rc" );
   createShellGUI ( true );
 
-  m_pluginManager->enableAllPluginsGUI (this);
+  KatePluginManager::self()->enableAllPluginsGUI (this);
 
   // connect settings menu aboutToshow
   documentMenu = (QPopupMenu*)factory()->container("documents", this);
   connect(documentMenu, SIGNAL(aboutToShow()), this, SLOT(documentMenuAboutToShow()));
 
-  connect(m_projectManager->projectManager(),SIGNAL(projectDeleted(uint)),this,SLOT(projectDeleted(uint)));
+  connect(KateProjectManager::self()->projectManager(),SIGNAL(projectDeleted(uint)),this,SLOT(projectDeleted(uint)));
 
   // caption update
-  for (uint i = 0; i < m_docManager->documents(); i++)
-    slotDocumentCreated (m_docManager->document(i));
+  for (uint i = 0; i < KateDocManager::self()->documents(); i++)
+    slotDocumentCreated (KateDocManager::self()->document(i));
 
-  connect(m_docManager,SIGNAL(documentCreated(Kate::Document *)),this,SLOT(slotDocumentCreated(Kate::Document *)));
+  connect(KateDocManager::self(),SIGNAL(documentCreated(Kate::Document *)),this,SLOT(slotDocumentCreated(Kate::Document *)));
 
   readOptions(config);
 
@@ -161,8 +160,8 @@ KateMainWindow::KateMainWindow(KateDocManager *_m_docManager, KatePluginManager
   setAcceptDrops(true);
 
   // activate the first restored project, if any
-  if (m_projectManager->projects() > 0)
-    activateProject(m_projectManager->project(0));
+  if (KateProjectManager::self()->projects() > 0)
+    activateProject(KateProjectManager::self()->project(0));
   else
     activateProject(0);
 }
@@ -173,7 +172,7 @@ KateMainWindow::~KateMainWindow()
 
   ((KateApp *)kapp)->removeMainWindow (this);
 
-  m_pluginManager->disableAllPluginsGUI (this);
+  KatePluginManager::self()->disableAllPluginsGUI (this);
 
   delete m_dcop;
   delete kscript;
@@ -188,31 +187,23 @@ void KateMainWindow::setupMainWindow ()
   greptool->show();
   greptool->hide();
 
-  KMdiChildView* pMDICover = new KMdiChildView("MainDock");
-  pMDICover->setName("MainDock");
+  m_viewManager = new KateViewManager (this);
 
-  //mainDock->setGeometry(100, 100, 100, 100);
-  QBoxLayout *ml=new QHBoxLayout(pMDICover);
-  ml->setAutoAdd(true);
-  m_viewManager = new KateViewManager (pMDICover, m_docManager,this);
-  addWindow(pMDICover);
-  m_viewManager->show();
-  pMDICover->show();
-
-  filelist = new KateFileList (m_docManager, m_viewManager, this/*filelistDock*/, "filelist");
-  addToolView(KDockWidget::DockLeft,filelist,SmallIcon("kmultiple"), i18n("Files"));
+  filelist = new KateFileList (this, m_viewManager, this/*filelistDock*/, "filelist");
+  filelist->readConfig(kapp->config(), "Filelist");
+  addToolView(KDockWidget::DockLeft,filelist,SmallIcon("kmultiple"), i18n("Documents"));
 
   QVBox *prBox = new QVBox (this,"projects");
   addToolView(KDockWidget::DockLeft,prBox,SmallIcon("view_tree"), i18n("Projects"));
-  projectlist = new KateProjectList (m_projectManager, this, prBox/*filelistDock*/, "projectlist");
-  projectviews = new KateProjectViews (m_projectManager, this, prBox/*filelistDock*/, "projectviews");
+  projectlist = new KateProjectList (this, prBox/*filelistDock*/, "projectlist");
+  projectviews = new KateProjectViews (this, prBox/*filelistDock*/, "projectviews");
   prBox->setStretchFactor(projectviews, 2);
   prBox->show ();
   projectlist->show ();
   projectviews->show ();
 
   fileselector = new KateFileSelector( this, m_viewManager, /*fileselectorDock*/ this, "operator");
-  addToolView(KDockWidget::DockLeft,fileselector, SmallIcon("fileopen"), i18n("Selector"));
+  addToolView(KDockWidget::DockLeft,fileselector, SmallIcon("fileopen"), i18n("Filesystem Browser"));
 
   // TEST
   addToolView( KDockWidget::DockBottom, greptool, SmallIcon("filefind"), i18n("Find in Files") );
@@ -231,7 +222,7 @@ void KateMainWindow::setupActions()
   KAction *a;
 
   kscript = new KScriptManager(this, "scriptmanager");
-  scriptMenu = new KActionMenu( i18n("KDE Scri&pts"), actionCollection(), "scripts");
+  scriptMenu = new KActionMenu( i18n("&KDE Scripts"), actionCollection(), "scripts");
   scriptMenu->setWhatsThis(i18n("This shows all available scripts and allows them to be executed."));
   setupScripts();
   connect( scriptMenu->popupMenu(), SIGNAL(activated( int)), this, SLOT(runScript( int )) );
@@ -242,12 +233,12 @@ void KateMainWindow::setupActions()
   fileOpenRecent = KStdAction::openRecent (m_viewManager, SLOT(openURL (const KURL&)), actionCollection());
   fileOpenRecent->setWhatsThis(i18n("This lists files which you have opened recently, and allows you to easily open them again."));
 
-  a=new KAction( i18n("Save A&ll"),"save_all", CTRL+Key_L, m_viewManager, SLOT( slotDocumentSaveAll() ), actionCollection(), "file_save_all" );
-  a->setWhatsThis(i18n("Save all open, modified documents to disc."));
+  a=new KAction( i18n("Save A&ll"),"save_all", CTRL+Key_L, KateDocManager::self(), SLOT( saveAll() ), actionCollection(), "file_save_all" );
+  a->setWhatsThis(i18n("Save all open, modified documents to disk."));
 
   KStdAction::close( m_viewManager, SLOT( slotDocumentClose() ), actionCollection(), "file_close" )->setWhatsThis(i18n("Close the current document."));
 
-  a=new KAction( i18n( "Clos&e All" ), 0, m_viewManager, SLOT( slotDocumentCloseAll() ), actionCollection(), "file_close_all" );
+  a=new KAction( i18n( "Clos&e All" ), 0, this, SLOT( slotDocumentCloseAll() ), actionCollection(), "file_close_all" );
   a->setWhatsThis(i18n("Close all open documents."));
 
   KStdAction::mail( this, SLOT(slotMail()), actionCollection() )->setWhatsThis(i18n("Send one or more of the open documents as email attachments."));
@@ -257,30 +248,12 @@ void KateMainWindow::setupActions()
   a=new KAction(i18n("&New Window"), "window_new", 0, this, SLOT(newWindow()), actionCollection(), "view_new_view");
   a->setWhatsThis(i18n("Create a new Kate view (a new window with the same document list)."));
 
-  a=new KAction( i18n("Split &Vertical"), "view_left_right", CTRL+SHIFT+Key_L, m_viewManager, SLOT( slotSplitViewSpaceVert() ), actionCollection(), "view_split_vert");
-  a->setWhatsThis(i18n("Split the currently active view vertically into two views."));
-
-  a=new KAction( i18n("Split &Horizontal"), "view_top_bottom", CTRL+SHIFT+Key_T, m_viewManager, SLOT( slotSplitViewSpaceHoriz() ), actionCollection(), "view_split_horiz");
-  a->setWhatsThis(i18n("Split the currently active view horizontally into two views."));
-
-  a=closeCurrentViewSpace = new KAction( i18n("Close &Current View"), "view_remove", CTRL+SHIFT+Key_R, m_viewManager, SLOT( slotCloseCurrentViewSpace() ), actionCollection(), "view_close_current_space");
-  a->setWhatsThis(i18n("Close the currently active splitted view"));
-
   externalTools = new KateExternalToolsMenuAction( i18n("External Tools"), actionCollection(), "tools_external", this );
   externalTools->setWhatsThis( i18n("Launch external helper applications") );
 
   showFullScreenAction = KStdAction::fullScreen( 0, 0, actionCollection(),this);
   connect( showFullScreenAction,SIGNAL(toggled(bool)), this,SLOT(slotFullScreen(bool)));
 
-  goNext=new KAction(i18n("Next View"),Key_F8,m_viewManager, SLOT(activateNextView()),actionCollection(),"go_next");
-  goNext->setWhatsThis(i18n("Make the next split view the active one."));
-
-  goPrev=new KAction(i18n("Previous View"),SHIFT+Key_F8,m_viewManager, SLOT(activatePrevView()),actionCollection(),"go_prev");
-  goPrev->setWhatsThis(i18n("Make the previous split view the active one."));
-
-  windowNext = KStdAction::back(filelist, SLOT(slotPrevDocument()), actionCollection());
-  windowPrev = KStdAction::forward(filelist, SLOT(slotNextDocument()), actionCollection());
-
   documentOpenWith = new KActionMenu(i18n("Open W&ith"), actionCollection(), "file_open_with");
   documentOpenWith->setWhatsThis(i18n("Open the current document using another application registered for its file type, or an application of your choice."));
   connect(documentOpenWith->popupMenu(), SIGNAL(aboutToShow()), this, SLOT(mSlotFixOpenWithMenu()));
@@ -292,7 +265,6 @@ void KateMainWindow::setupActions()
   a=KStdAction::configureToolbars(this, SLOT(slotEditToolbars()), actionCollection(), "set_configure_toolbars");
   a->setWhatsThis(i18n("Configure which items should appear in the toolbar(s)."));
 
-
   // project menu
   a = new KAction(i18n("&New Project..."), "filenew", 0, this, SLOT(slotProjectNew()), actionCollection(), "project_new");
   a = new KAction(i18n("&Open Project..."), "fileopen", 0, this, SLOT(slotProjectOpen()), actionCollection(), "project_open");
@@ -311,18 +283,46 @@ void KateMainWindow::setupActions()
   // tip of the day :-)
   KStdAction::tipOfDay( this, SLOT( tipOfTheDay() ), actionCollection() )->setWhatsThis(i18n("This shows useful tips on the use of this application."));
 
-  if (m_pluginManager->pluginList().count() > 0)
+  if (KatePluginManager::self()->pluginList().count() > 0)
   {
-    a=new KAction(i18n("Contents &Plugins"), 0, this, SLOT(pluginHelp()), actionCollection(), "help_plugins_contents");
+    a=new KAction(i18n("&Plugins Handbook"), 0, this, SLOT(pluginHelp()), actionCollection(), "help_plugins_contents");
     a->setWhatsThis(i18n("This shows help files for various available plugins."));
   }
 
   connect(m_viewManager,SIGNAL(viewChanged()),this,SLOT(slotWindowActivated()));
   connect(m_viewManager,SIGNAL(viewChanged()),this,SLOT(slotUpdateOpenWith()));
-  connect(m_docManager,SIGNAL(documentChanged()),this,SLOT(slotDocumentChanged()));
 
   slotWindowActivated ();
-  slotDocumentChanged();
+
+  projectlist->setupActions();
+}
+
+void KateMainWindow::slotDocumentCloseAll() {
+  if (queryClose_internal())
+    KateDocManager::self()->closeAllDocuments(false);
+}
+
+bool KateMainWindow::queryClose_internal() {
+   uint documentCount=KateDocManager::self()->documents();
+
+  if ( ! showModOnDiskPrompt() )
+    return false;
+
+  QPtrList<Kate::Document> modifiedDocuments=KateDocManager::self()->modifiedDocumentList();
+  bool shutdown=(modifiedDocuments.count()==0);
+
+  if (!shutdown) {
+    shutdown=KateSaveModifiedDialog::queryClose(this,modifiedDocuments);
+  }
+
+  if ( KateDocManager::self()->documents() > documentCount ) {
+    KMessageBox::information (this,
+                              i18n ("New file opened while trying to close Kate, closing aborted."),
+                              i18n ("Closing Aborted"));
+    shutdown=false;
+  }
+
+  return shutdown;
 }
 
 /**
@@ -330,14 +330,12 @@ void KateMainWindow::setupActions()
  */
 bool KateMainWindow::queryClose()
 {
-  kdDebug(13000)<<"QUERY CLOSE ********************"<<endl;
-
   // session saving, can we close all projects & views ?
   // just test, not close them actually
   if (kapp->sessionSaving())
   {
-    return ( m_projectManager->queryCloseAll () &&
-             m_docManager->queryCloseDocuments (this) );
+    return ( KateProjectManager::self()->queryCloseAll () &&
+             queryClose_internal() );
   }
 
   // normal closing of window
@@ -347,8 +345,10 @@ bool KateMainWindow::queryClose()
 
   // last one: check if we can close all projects/document, try run
   // and save projects/docs if we really close down !
-  if ( m_projectManager->queryCloseAll () &&
-       m_docManager->queryCloseDocuments (this) )
+
+
+  if ( KateProjectManager::self()->queryCloseAll () &&
+       queryClose_internal() )
   {
     KConfig scfg("katesessionrc", false);
 
@@ -356,10 +356,10 @@ bool KateMainWindow::queryClose()
     config->setGroup("General");
 
     if (config->readBoolEntry("Restore Projects", false))
-      m_projectManager->saveProjectList (&scfg);
+      KateProjectManager::self()->saveProjectList (&scfg);
 
     if (config->readBoolEntry("Restore Documents", false))
-      m_docManager->saveDocumentList (&scfg);
+      KateDocManager::self()->saveDocumentList (&scfg);
 
     if (config->readBoolEntry("Restore Window Configuration", false))
       saveProperties (&scfg);
@@ -389,23 +389,18 @@ void KateMainWindow::slotFileQuit()
 
 void KateMainWindow::readOptions(KConfig *config)
 {
-  applyMainWindowSettings(config, "Kate Main Window");
-
   config->setGroup("General");
   syncKonsole =  config->readBoolEntry("Sync Konsole", true);
   modNotification = config->readBoolEntry("Modified Notification", false);
-  m_docManager->setSaveMetaInfos(config->readBoolEntry("Save Meta Infos", true));
-  m_docManager->setDaysMetaInfos(config->readNumEntry("Days Meta Infos", 30));
+  KateDocManager::self()->setSaveMetaInfos(config->readBoolEntry("Save Meta Infos", true));
+  KateDocManager::self()->setDaysMetaInfos(config->readNumEntry("Days Meta Infos", 30));
 
   m_viewManager->setShowFullPath(config->readBoolEntry("Show Full Path in Title", false));
 
-  fileOpenRecent->setMaxItems( config->readNumEntry("Number of recent files", fileOpenRecent->maxItems() ) );
   fileOpenRecent->loadEntries(config, "Recent Files");
 
   fileselector->readConfig(config, "fileselector");
 
-  filelist->setSortType(config->readNumEntry("Sort Type of File List", KateFileList::sortByID));
-
   recentProjects->loadEntries (config, "Recent Projects");
 }
 
@@ -420,9 +415,9 @@ void KateMainWindow::saveOptions(KConfig *config)
   else
     config->writeEntry("Show Console", false);
 
-  config->writeEntry("Save Meta Infos", m_docManager->getSaveMetaInfos());
+  config->writeEntry("Save Meta Infos", KateDocManager::self()->getSaveMetaInfos());
 
-  config->writeEntry("Days Meta Infos", m_docManager->getDaysMetaInfos());
+  config->writeEntry("Days Meta Infos", KateDocManager::self()->getDaysMetaInfos());
 
   config->writeEntry("Show Full Path in Title", m_viewManager->getShowFullPath());
 
@@ -432,25 +427,11 @@ void KateMainWindow::saveOptions(KConfig *config)
 
   fileselector->writeConfig(config, "fileselector");
 
-  config->writeEntry("Sort Type of File List", filelist->sortType());
+  filelist->writeConfig(config, "Filelist");
 
   recentProjects->saveEntries (config, "Recent Projects");
 }
 
-void KateMainWindow::slotDocumentChanged()
-{
-  if (m_docManager->documents()  > 1)
-  {
-    windowNext->setEnabled(true);
-    windowPrev->setEnabled(true);
-  }
-  else
-  {
-    windowNext->setEnabled(false);
-    windowPrev->setEnabled(false);
-  }
-}
-
 void KateMainWindow::slotWindowActivated ()
 {
   static QString path;
@@ -470,11 +451,6 @@ void KateMainWindow::slotWindowActivated ()
 
     updateCaption (m_viewManager->activeView()->getDoc());
   }
-
-  if (m_viewManager->viewSpaceCount() == 1)
-    closeCurrentViewSpace->setEnabled(false);
-  else
-    closeCurrentViewSpace->setEnabled(true);
 }
 
 void KateMainWindow::slotUpdateOpenWith()
@@ -488,19 +464,18 @@ void KateMainWindow::slotUpdateOpenWith()
 void KateMainWindow::documentMenuAboutToShow()
 {
   documentMenu->clear ();
-  windowNext->plug (documentMenu);
-  windowPrev->plug (documentMenu);
-  documentMenu->insertSeparator ();
 
-  for (uint z=0; z < filelist->count(); z++)
-  {
-    documentMenu->insertItem (filelist->item(z)->text(),
-                              m_viewManager, SLOT (activateView (int)), 0,
-                              ((KateFileListItem *)filelist->item (z))->documentNumber ());
+  QListViewItem * item = filelist->firstChild();
+  while( item ) {
+    documentMenu->insertItem (
+	  KStringHandler::rsqueeze( ((KateFileListItem *)item)->document()->docName(), 150 ), // would it be saner to use the screen width as a limit that some random number??
+          m_viewManager, SLOT (activateView (int)), 0,
+	  ((KateFileListItem *)item)->documentNumber () );
 
-    if (m_viewManager->activeView())
-      documentMenu->setItemChecked ( m_viewManager->activeView()->getDoc()->documentNumber(), true);
+    item = item->nextSibling();
   }
+  if (m_viewManager->activeView())
+    documentMenu->setItemChecked ( m_viewManager->activeView()->getDoc()->documentNumber(), true);
 }
 
 void KateMainWindow::slotGrepToolItemSelected(const QString &filename,int linenumber)
@@ -510,8 +485,8 @@ void KateMainWindow::slotGrepToolItemSelected(const QString &filename,int linenu
   m_viewManager->openURL( fileURL );
   if ( m_viewManager->activeView() == 0 ) return;
   m_viewManager->activeView()->gotoLineNumber( linenumber );
-  this->raise();
-  this->setActiveWindow();
+  raise();
+  setActiveWindow();
 }
 
 void KateMainWindow::dragEnterEvent( QDragEnterEvent *event )
@@ -548,17 +523,15 @@ void KateMainWindow::editKeys()
 
   dlg.configure();
 
-  QPtrList<Kate::Document>  l=m_docManager->documentList();
+  QPtrList<Kate::Document>  l=KateDocManager::self()->documentList();
   for (uint i=0;i<l.count();i++) {
-	kdDebug(13001)<<"reloading Keysettings for document "<<i<<endl;
-	l.at(i)->reloadXML();
-	QPtrList<class KTextEditor::View> l1=l.at(i)->views ();//KTextEditor::Document
-	for (uint i1=0;i1<l1.count();i1++) {
-		l1.at(i1)->reloadXML();
-		kdDebug(13001)<<"reloading Keysettings for view "<<i<<"/"<<i1<<endl;
-
-	}
-
+//     kdDebug(13001)<<"reloading Keysettings for document "<<i<<endl;
+    l.at(i)->reloadXML();
+    QPtrList<class KTextEditor::View> l1=l.at(i)->views ();//KTextEditor::Document
+    for (uint i1=0;i1<l1.count();i1++) {
+      l1.at(i1)->reloadXML();
+//       kdDebug(13001)<<"reloading Keysettings for view "<<i<<"/"<<i1<<endl;
+    }
   }
 
   externalTools->actionCollection()->writeShortcutSettings( "Shortcuts", new KConfig("externaltools", false, false, "appdata") );
@@ -580,22 +553,6 @@ void KateMainWindow::slotConfigure()
   delete dlg;
 }
 
-//Set focus to next input element
-void KateMainWindow::slotGoNext()
-{
-  QFocusEvent::setReason(QFocusEvent::Tab);
-  /*res= */focusNextPrevChild(true); //TRUE == NEXT , FALSE = PREV
-  QFocusEvent::resetReason();
-}
-
-//Set focus to previous input element
-void KateMainWindow::slotGoPrev()
-{
-  QFocusEvent::setReason(QFocusEvent::Tab);
-  /*res= */focusNextPrevChild(false); //TRUE == NEXT , FALSE = PREV
-  QFocusEvent::resetReason();
-}
-
 KURL KateMainWindow::activeDocumentUrl()
 {
   // anders: i make this one safe, as it may be called during
@@ -679,8 +636,8 @@ void KateMainWindow::setupScripts()
 
 void KateMainWindow::runScript( int mIId )
 {
-	//kdDebug(13000) << "Starting script engine..." << endl;
-        kdDebug(13001)<<"runScript( "<<mIId<<" ) ["<<scriptMenu->popupMenu()->text( mIId )<<"]"<<endl;
+	//kdDebug(13001) << "Starting script engine..." << endl;
+//         kdDebug(13001)<<"runScript( "<<mIId<<" ) ["<<scriptMenu->popupMenu()->text( mIId )<<"]"<<endl;
 	kscript->runScript( scriptMenu->popupMenu()->text( mIId ) );
 }
 
@@ -771,55 +728,72 @@ void KateMainWindow::slotFullScreen(bool t)
 
 bool KateMainWindow::eventFilter( QObject *o, QEvent *e )
 {
-  if ( e->type() == QEvent::WindowActivate && o == this ) {
-    Kate::Document *doc;
-    typedef QPtrVector<Kate::Document> docvector;
-    docvector list( m_docManager->documents() );
-    uint cnt = 0;
-    for( doc = m_docManager->firstDocument(); doc; doc = m_docManager->nextDocument() )
-    {
-      if ( m_docManager->documentInfo( doc )->modifiedOnDisc )
-      {
-        list.insert( cnt, doc );
-        cnt++;
-      }
-    }
 
-    if ( cnt )
-    {
-      list.resize( cnt );
-
-      // TODO
-      // display a dialog with a list of modified documents,
-      // and options to reload/disguard all, or handle individually
-      for ( uint i=0; i < cnt; i++ )
-      {
-        Kate::DocumentExt *ext = Kate::documentExt (list.at( i ));
-
-        if (ext)
-          ext->slotModifiedOnDisk( activeView );
-      }
-    }
-  }
-
-  if ( o == greptool && e->type() == QEvent::Show && activeView )
+  if ( o == greptool && e->type() == QEvent::Show && m_viewManager->activeView() )
   {
-    if ( activeView->getDoc()->url().isLocalFile() )
+    if ( m_viewManager->activeView()->getDoc()->url().isLocalFile() )
     {
-      greptool->updateDirName( activeView->getDoc()->url().directory() );
+      greptool->updateDirName( m_viewManager->activeView()->getDoc()->url().directory() );
       return true;
     }
   }
   if ( ( o == greptool || o == console ) &&
-      e->type() == QEvent::Hide && activeView )
+      e->type() == QEvent::Hide && m_viewManager->activeView() )
   {
-     activeView->setFocus();
+     m_viewManager->activeView()->setFocus();
      return true;
   }
-  return KMdiMainFrm::eventFilter( o, e );
+
+  return KMDI::MainWindow::eventFilter( o, e );
 }
 
-KMdiToolViewAccessor *KateMainWindow::addToolView(KDockWidget::DockPosition position, QWidget *widget, const QPixmap &icon, const QString &sname, const QString &tabToolTip, const QString &tabCaption)
+bool KateMainWindow::event( QEvent *e )
+{
+  uint type = e->type();
+  if ( type == QEvent::WindowActivate && modNotification )
+  {
+    if ( m_modignore )
+    {
+      m_modignore = false;
+      return KMDI::MainWindow::event( e );
+    }
+    showModOnDiskPrompt();
+  }
+  // Try to disable the modonhd prompt from showing after internal dialogs.
+  // TODO make this work better.
+  else if ( (type == QEvent::WindowUnblocked || type == QEvent::WindowBlocked) && modNotification)
+    m_modignore = true;
+
+  return KMDI::MainWindow::event( e );
+}
+
+bool KateMainWindow::showModOnDiskPrompt()
+{
+  Kate::Document *doc;
+
+  DocVector list( KateDocManager::self()->documents() );
+  uint cnt = 0;
+  for( doc = KateDocManager::self()->firstDocument(); doc; doc = KateDocManager::self()->nextDocument() )
+  {
+    if ( KateDocManager::self()->documentInfo( doc )->modifiedOnDisc )
+    {
+      list.insert( cnt, doc );
+      cnt++;
+    }
+  }
+
+  if ( cnt )
+  {
+    list.resize( cnt );
+    KateMwModOnHdDialog mhdlg( list, this );
+    bool res = mhdlg.exec();
+
+    return res;
+  }
+  return true;
+}
+
+KMDI::ToolViewAccessor *KateMainWindow::addToolView(KDockWidget::DockPosition position, QWidget *widget, const QPixmap &icon, const QString &sname, const QString &tabToolTip, const QString &tabCaption)
 {
   widget->setIcon(icon);
   widget->setCaption(sname);
@@ -833,21 +807,21 @@ bool KateMainWindow::removeToolView(QWidget *w)
   return true;
 }
 
-bool KateMainWindow::removeToolView(KMdiToolViewAccessor *accessor)
+bool KateMainWindow::removeToolView(KMDI::ToolViewAccessor *accessor)
 {
   deleteToolWindow (accessor);
   return true;
 }
 
 bool KateMainWindow::showToolView(QWidget *){return false;}
-bool KateMainWindow::showToolView(KMdiToolViewAccessor *){return false;}
+bool KateMainWindow::showToolView(KMDI::ToolViewAccessor *){return false;}
 
 bool KateMainWindow::hideToolView(QWidget *){return false;}
-bool KateMainWindow::hideToolView(KMdiToolViewAccessor *){return false;}
+bool KateMainWindow::hideToolView(KMDI::ToolViewAccessor *){return false;}
 
 void KateMainWindow::slotProjectNew ()
 {
-  ProjectInfo *info = m_projectManager->newProjectDialog (this);
+  ProjectInfo *info = KateProjectManager::self()->newProjectDialog (this);
 
   if (info)
   {
@@ -874,24 +848,24 @@ void KateMainWindow::slotProjectClose ()
 {
   if (m_project)
   {
-    m_projectManager->close (m_project);
+    KateProjectManager::self()->close (m_project);
   }
 }
 
 void KateMainWindow::activateProject (Kate::Project *project)
 {
-  kdDebug(13001)<<"activating project "<<project<<endl;
+//   kdDebug(13001)<<"activating project "<<project<<endl;
   if (m_project)
-    m_projectManager->disableProjectGUI (m_project, this);
+    KateProjectManager::self()->disableProjectGUI (m_project, this);
 
   if (project)
-    m_projectManager->enableProjectGUI (project, this);
+    KateProjectManager::self()->enableProjectGUI (project, this);
 
   m_project = project;
 
   if (project)
   {
-    m_projectManager->setCurrentProject (project);
+    KateProjectManager::self()->setCurrentProject (project);
     m_projectNumber = project->projectNumber ();
   }
   else
@@ -905,7 +879,7 @@ void KateMainWindow::activateProject (Kate::Project *project)
 
 Kate::Project *KateMainWindow::createProject (const QString &type, const QString &name, const QString &filename)
 {
-  Kate::Project *project = m_projectManager->create (type, name, filename);
+  Kate::Project *project = KateProjectManager::self()->create (type, name, filename);
 
   if (project)
     activateProject (project);
@@ -915,7 +889,7 @@ Kate::Project *KateMainWindow::createProject (const QString &type, const QString
 
 Kate::Project *KateMainWindow::openProject (const QString &filename)
 {
-  Kate::Project *project = m_projectManager->open (filename);
+  Kate::Project *project = KateProjectManager::self()->open (filename);
 
   if (project)
   {
@@ -930,8 +904,8 @@ void KateMainWindow::projectDeleted (uint projectNumber)
 {
   if (projectNumber == m_projectNumber)
   {
-    if (m_projectManager->projects() > 0)
-      activateProject (m_projectManager->project(m_projectManager->projects()-1));
+    if (KateProjectManager::self()->projects() > 0)
+      activateProject (KateProjectManager::self()->project(KateProjectManager::self()->projects()-1));
     else
       activateProject (0);
   }
@@ -957,28 +931,17 @@ void KateMainWindow::updateCaption (Kate::Document *doc)
   if (!(m_viewManager->activeView()->getDoc() == doc))
     return;
 
-  // update the sync action
-  fileselector->kateViewChanged();
-
   QString c;
   if (m_viewManager->activeView()->getDoc()->url().isEmpty() || (!m_viewManager->getShowFullPath()))
   {
     c = m_viewManager->activeView()->getDoc()->docName();
-
-    //File name shouldn't be too long - Maciek
-    if (c.length() > 64)
-      c = c.left(64) + "...";
   }
   else
   {
     c = m_viewManager->activeView()->getDoc()->url().prettyURL();
-
-    //File name shouldn't be too long - Maciek
-    if (c.length() > 64)
-      c = "..." + c.right(64);
   }
 
-  setCaption( c, m_viewManager->activeView()->getDoc()->isModified());
+  setCaption( KStringHandler::lsqueeze(c,64), m_viewManager->activeView()->getDoc()->isModified());
 }
 
 void KateMainWindow::openConstURLProject (const KURL&url)
@@ -987,12 +950,10 @@ void KateMainWindow::openConstURLProject (const KURL&url)
 }
 
 void KateMainWindow::saveProperties(KConfig *config) {
-  kdDebug(13000)<<"KateMainWindow::saveProperties()**********************"<<endl
-  <<config->group()<<endl
-  <<"******************************************************"<<endl;
+  kdDebug(13000)<<"KateMainWindow::saveProperties(): group: "<<config->group()<<endl;
   assert(config);
 
-  kdDebug(13000)<<"preparing session saving"<<endl;
+//   kdDebug(13000)<<"preparing session saving"<<endl;
   QString grp=config->group();
   QString dockGrp;
 
@@ -1003,16 +964,16 @@ void KateMainWindow::saveProperties(KConfig *config) {
         config->deleteGroup("MainWindow0-Docking");
   }*/
 
-  kdDebug(13000)<<"Before write dock config"<<endl;
+//   kdDebug(13000)<<"Before write dock config"<<endl;
   writeDockConfig(config,dockGrp);
-  kdDebug(13000)<<"After write dock config"<<endl;
+//   kdDebug(13000)<<"After write dock config"<<endl;
 
 
   if (kapp->sessionSaving()) dockGrp=grp+"-View Configuration";
 	else dockGrp="MainWindow0-View Configuration";
 
   m_viewManager->saveViewConfiguration (config,dockGrp);
-  kdDebug(13000)<<"After saving view configuration"<<endl;
+//   kdDebug(13000)<<"After saving view configuration"<<endl;
   config->setGroup(grp);
 
 }
@@ -1037,8 +998,8 @@ void KateMainWindow::readProperties(KConfig *config)
 
 void KateMainWindow::saveGlobalProperties( KConfig* sessionConfig )
 {
-  m_projectManager->saveProjectList (sessionConfig);
-  m_docManager->saveDocumentList (sessionConfig);
+  KateProjectManager::self()->saveProjectList (sessionConfig);
+  KateDocManager::self()->saveDocumentList (sessionConfig);
 }
 
 void KateMainWindow::slotPipeToConsole ()
@@ -1056,3 +1017,4 @@ void KateMainWindow::slotPipeToConsole ()
   else
     console->sendInput (v->getDoc()->text());
 }
+// kate: space-indent on; indent-width 2; replace-tabs on;
