@@ -25,10 +25,12 @@
 namespace Kate
 {
 
+class View;
+
 /**
  * Interface to the viewmanager
  */
-class ViewManager : public QObject
+class KDE_EXPORT ViewManager : public QObject
 {
   friend class PrivateViewManager;
 
@@ -45,12 +47,18 @@ class ViewManager : public QObject
      */
     virtual ~ViewManager ();
       
-  public:
+  public slots: /*these are slots for kjs*/
     /**
      * Returns a pointer to the currently active view
      * @return View active view
      */
-    class View *activeView ();
+    Kate::View *activeView ();
+
+    /**
+     * Activates the view with the corresponding documentNumber
+     * @param documentNumber the document's number
+     */
+    void activateView ( uint documentNumber );
 
     /**
      * Opens the file pointed to by URL
