@@ -20,6 +20,7 @@
 #include "kdiconview.h"
 #include "krootwm.h"
 #include "desktop.h"
+#include "kdesktopsettings.h"
 
 #include <kaccel.h>
 #include <kapplication.h>
@@ -36,6 +37,7 @@
 #include <konq_popupmenu.h>
 #include <konq_settings.h>
 #include <konq_undo.h>
+#include <kprotocolinfo.h>
 #include <kstdaction.h>
 #include <kstandarddirs.h>
 #include <kurldrag.h>
@@ -63,8 +65,8 @@ do { \
     int x = _x; int y = _y; \
     x -= iconArea.x(); \
     y -= iconArea.y(); \
-    if (x > ((desk.width()*3) / 4)) x -= desk.width(); \
-    if (y > ((desk.height()*3) / 4)) y -= desk.height(); \
+    if (x > ((iconArea.width()*3) / 4)) x -= iconArea.width(); \
+    if (y > ((iconArea.height()*3) / 4)) y -= iconArea.height(); \
     (config)->writeEntry("X", x); (config)->writeEntry("Y", y); \
     (config)->writeEntry(X_w, x); (config)->writeEntry(Y_h, y); \
 } while(false)
@@ -73,10 +75,10 @@ do { \
 do { \
     x = (config)->readNumEntry( X_w, -9999 ); \
     if ( x == -9999 ) x = (config)->readNumEntry( "X" ); \
-    if (x < 0) x += desk.width(); \
+    if (x < 0) x += iconArea.width(); \
     y = (config)->readNumEntry( Y_h, -9999 ); \
     if ( y == -9999 ) y = (config)->readNumEntry( "Y" ); \
-    if (y < 0) y += desk.height(); \
+    if (y < 0) y += iconArea.height(); \
     x += iconArea.x(); \
     y += iconArea.y(); \
 } while(false)
@@ -94,6 +96,7 @@ KDIconView::KDIconView( QWidget *parent, const char* name )
       m_bNeedRepaint( false ),
       m_bNeedSave( false ),
       m_hasExistingPos( false ),
+      m_bEditableDesktopIcons( kapp->authorize("editable_desktop_icons") ),
       m_bShowDot( false ),
       m_bVertAlign( true ),
       m_dirLister( 0L ),
@@ -137,12 +140,11 @@ KDIconView::KDIconView( QWidget *parent, const char* name )
     connect( this, SIGNAL(itemRenamed(QIconViewItem *, const QString &)),
              this, SLOT(slotItemRenamed(QIconViewItem *, const QString &)) );
 
-    setShowToolTips( false );
-
-    if (!kapp->authorize("editable_desktop_icons"))
+    if (!m_bEditableDesktopIcons)
     {
        setItemsMovable(false);
        setAcceptDrops(false);
+       viewport()->setAcceptDrops(false);
     }
 
 #if (QT_VERSION-0 >= 0x030200) // XRANDR support
@@ -152,6 +154,9 @@ KDIconView::KDIconView( QWidget *parent, const char* name )
 
 KDIconView::~KDIconView()
 {
+    if (m_dotDirectory && !m_bEditableDesktopIcons)
+      m_dotDirectory->rollback(false); // Don't save positions
+
     delete m_dotDirectory;
     delete m_dirLister;
     delete m_shadowEngine;
@@ -164,16 +169,19 @@ void KDIconView::initDotDirectories()
     if (u.isLocalFile())
        dirs.prepend(u.path());
 
-    QString prefix = iconPositionGroupPrefix(); 
+    QString prefix = iconPositionGroupPrefix();
     QString dotFileName = locateLocal("appdata", "IconPositions");
     if (kdesktop_screen_number != 0)
        dotFileName += "_Desktop" + QString::number(kdesktop_screen_number);
 
+    if (m_dotDirectory && !m_bEditableDesktopIcons)
+      m_dotDirectory->rollback(false); // Don't save positions
+
     delete m_dotDirectory;
 
     m_dotDirectory = new KSimpleConfig( dotFileName );
     // If we don't allow editable desktop icons, empty m_dotDirectory
-    if (!kapp->authorize("editable_desktop_icons"))
+    if (!m_bEditableDesktopIcons)
     {
         QStringList groups = m_dotDirectory->groupList();
         QStringList::ConstIterator gIt = groups.begin();
@@ -190,11 +198,11 @@ void KDIconView::initDotDirectories()
     {
         kdDebug(1204) << "KDIconView::initDotDirectories found dir " << *it << endl;
         QString dotFileName = *it + "/.directory";
-        
+
         if (QFile::exists(dotFileName))
         {
            KSimpleConfig dotDir(dotFileName, true); // Read only
-           
+
            QStringList groups = dotDir.groupList();
            QStringList::ConstIterator gIt = groups.begin();
            QStringList::ConstIterator gEnd = groups.end();
@@ -204,7 +212,7 @@ void KDIconView::initDotDirectories()
               {
                  dotDir.setGroup( *gIt );
                  m_dotDirectory->setGroup( *gIt );
-                 
+
                  if (!m_dotDirectory->hasKey( X_w ))
                  {
                     int x,y;
@@ -222,8 +230,10 @@ void KDIconView::initConfig( bool init )
 {
     //kdDebug() << "initConfig " << init << endl;
 
-    if ( !init )
+    if ( !init ) {
         KonqFMSettings::reparseConfiguration();
+        KDesktopSettings::self()->readConfig();
+    }
 
     KConfig * config = KGlobal::config();
 
@@ -233,26 +243,27 @@ void KDIconView::initConfig( bool init )
     }
 
     setMaySetWallpaper(!config->isImmutable() && !KGlobal::dirs()->isRestrictedResource("wallpaper"));
-    config->setGroup( "Desktop Icons" );
-    m_bShowDot = config->readBoolEntry("ShowHidden", DEFAULT_SHOW_HIDDEN_ROOT_ICONS);
-    m_bVertAlign = config->readBoolEntry("VertAlign", DEFAULT_VERT_ALIGN);
+    m_bShowDot = KDesktopSettings::showHidden();
+    m_bVertAlign = KDesktopSettings::vertAlign();
     QStringList oldPreview = previewSettings();
-    setPreviewSettings( config->readListEntry( "Preview" ) );
+    setPreviewSettings( KDesktopSettings::preview() );
 
     // read arrange configuration
-    m_eSortCriterion  = (SortCriterion)config->readNumEntry("SortCriterion", NameCaseInsensitive);
-    m_bSortDirectoriesFirst = config->readBoolEntry("DirectoriesFirst", true);
-    m_itemsAlwaysFirst = config->readListEntry("AlwaysFirstItems"); // Distributor plug-in
+    m_eSortCriterion  = (SortCriterion)KDesktopSettings::sortCriterion();
+    m_bSortDirectoriesFirst = KDesktopSettings::directoriesFirst();
+    m_itemsAlwaysFirst = KDesktopSettings::alwaysFirstItems(); // Distributor plug-in
 
-    config->setGroup( "Devices" );
-    m_enableDevices=config->readBoolEntry("enabled",false);
-    QString tmpList=config->readEntry("exclude","kdedevice/hdd_mounted,kdedevice/hdd_unmounted,kdedevice/floppy_unmounted,kdedevice/cdrom_unmounted,kdedevice/floppy5_unmounted");
+    if (KProtocolInfo::isKnownProtocol(QString::fromLatin1("media")))
+        m_enableMedia=KDesktopSettings::mediaEnabled();
+    else
+        m_enableMedia=false;
+    QString tmpList=KDesktopSettings::exclude();
     kdDebug(1204)<<"m_excludeList"<<tmpList<<endl;
-    m_excludedDevices=QStringList::split(",",tmpList,false);
-    kdDebug(1204)<<" m_excludeList / item count:" <<m_excludedDevices.count()<<endl;
+    m_excludedMedia=QStringList::split(",",tmpList,false);
+    kdDebug(1204)<<" m_excludeList / item count:" <<m_excludedMedia.count()<<endl;
     if ( m_dirLister ) // only when called while running - not on first startup
     {
-        configureDevices();
+        configureMedia();
         m_dirLister->setShowingDotFiles( m_bShowDot );
         m_dirLister->emitChanges();
     }
@@ -291,22 +302,6 @@ void KDIconView::initConfig( bool init )
 
     if ( !init )
         updateContents();
-
-    // Update display of "Delete" command if necessary
-    if (kapp->authorize("editable_desktop_icons"))
-    {
-        KAction *del = m_actionCollection.action( "del" );
-        KConfig config("konquerorrc", true, false);
-        config.setGroup( "FMSettings" );
-        if ( config.readBoolEntry( "ShowDeleteCommand", true ) )
-        {
-            if (!del)
-                (void) new KAction( i18n( "&Delete" ), "editdelete", SHIFT+Key_Delete, this,
-                 SLOT( slotDelete() ), &m_actionCollection, "del" );
-        }
-        else
-          delete del;
-     }
 }
 
 void KDIconView::start()
@@ -352,41 +347,36 @@ void KDIconView::start()
         kapp->allowURLAction("list", KURL(), u);
         m_dirLister->openURL( u, true );
     }
-    configureDevices();
+    configureMedia();
     createActions();
 }
 
-void KDIconView::configureDevices()
+void KDIconView::configureMedia()
 {
-#if !(defined(Q_OS_LINUX) || defined(Q_OS_FREEBSD))
- m_enableDevices=false;
- return;
-#endif
-
-    kdDebug(1204) << "***********KDIconView::configureDevices() " <<endl;
-    m_dirLister->setMimeExcludeFilter(m_excludedDevices);
+    kdDebug(1204) << "***********KDIconView::configureMedia() " <<endl;
+    m_dirLister->setMimeExcludeFilter(m_excludedMedia);
     m_dirLister->emitChanges();
     updateContents();
-    if (m_enableDevices)
+    if (m_enableMedia)
     {
     	for (KURL::List::Iterator it1=m_mergeDirs.begin();it1!=m_mergeDirs.end();++it1)
 	    {
-	    	if ((*it1).url()=="devices:/") return;
+	    	if ((*it1).url()=="media:/") return;
 	    }
-    	m_mergeDirs.append(KURL("devices:/"));
-    	m_dirLister->openURL(KURL("devices:/"),true);
+    	m_mergeDirs.append(KURL("media:/"));
+    	m_dirLister->openURL(KURL("media:/"),true);
     }
     else
     {
             for (KURL::List::Iterator it2=m_mergeDirs.begin();it2!=m_mergeDirs.end();++it2)
 	    {
-		if ((*it2).url()=="devices:/")
+		if ((*it2).url()=="media:/")
 		{
 			  delete m_dirLister;
 			  m_dirLister=0;
 			  start();
 //			m_mergeDirs.remove(it2);
-//			m_dirLister->stop("devices");
+//			m_dirLister->stop("media");
 			return;
 		}
 
@@ -398,7 +388,7 @@ void KDIconView::configureDevices()
 
 void KDIconView::createActions()
 {
-    if (kapp->authorize("editable_desktop_icons"))
+    if (m_bEditableDesktopIcons)
     {
         KAction *undo = KStdAction::undo( KonqUndoManager::self(), SLOT( undo() ), &m_actionCollection, "undo" );
         connect( KonqUndoManager::self(), SIGNAL( undoAvailable( bool ) ),
@@ -407,18 +397,23 @@ void KDIconView::createActions()
              undo, SLOT( setText( const QString & ) ) );
         undo->setEnabled( KonqUndoManager::self()->undoAvailable() );
 
-        KStdAction::cut( this, SLOT( slotCut() ), &m_actionCollection, "cut" );
+        KAction* paCut = KStdAction::cut( this, SLOT( slotCut() ), &m_actionCollection, "cut" );
+        KShortcut cutShortCut = paCut->shortcut();
+        cutShortCut.remove( KKey( SHIFT + Key_Delete ) ); // used for deleting files
+        paCut->setShortcut( cutShortCut );
+
         KStdAction::copy( this, SLOT( slotCopy() ), &m_actionCollection, "copy" );
         KStdAction::paste( this, SLOT( slotPaste() ), &m_actionCollection, "paste" );
         KAction *pasteTo = KStdAction::paste( this, SLOT( slotPopupPasteTo() ), &m_actionCollection, "pasteto" );
         pasteTo->setEnabled( false ); // only enabled during popupMenu()
 
         (void) new KAction( i18n( "&Rename" ), /*"editrename",*/ Key_F2, this, SLOT( renameSelectedItem() ), &m_actionCollection, "rename" );
-        (void) new KAction( i18n( "&Move to Trash" ), "edittrash", Key_Delete, this, SLOT( slotTrash() ), &m_actionCollection, "trash" );
+        KAction* trash = new KAction( i18n( "&Move to Trash" ), "edittrash", Key_Delete, &m_actionCollection, "trash" );
+        connect( trash, SIGNAL( activated( KAction::ActivationReason, Qt::ButtonState ) ),
+                 this, SLOT( slotTrashActivated( KAction::ActivationReason, Qt::ButtonState ) ) );
 
-        KConfig config("konquerorrc", true, false);
-        config.setGroup( "FMSettings" );
-        if ( config.readBoolEntry( "ShowDeleteCommand", true ) )
+        KConfig config("kdeglobals", true, false);
+        config.setGroup( "KDE" );
         (void) new KAction( i18n( "&Delete" ), "editdelete", SHIFT+Key_Delete, this, SLOT( slotDelete() ), &m_actionCollection, "del" );
 
         // Initial state of the actions (cut/copy/paste/...)
@@ -463,6 +458,11 @@ void KDIconView::setAutoAlign( bool b )
                  this, SLOT( lineupIcons() ) );
     }
     else {
+        // change maxItemWidth, because when grid-align was active, it changed this for the grid
+        int sz = iconSize() ? iconSize() : KGlobal::iconLoader()->currentSize( KIcon::Desktop );
+        setMaxItemWidth( QMAX( QMAX( sz, previewIconSize( iconSize() ) ), KonqFMSettings::settings()->iconTextWidth() ) );
+        setFont( font() );  // Force calcRect()
+
         disconnect( this, SIGNAL( iconMoved() ),
                     this, SLOT( lineupIcons() ) );
     }
@@ -487,7 +487,7 @@ void KDIconView::desktopResized()
         kapp->allowURLAction("list", KURL(), u);
         m_dirLister->openURL( u, true );
     }
-    configureDevices();
+    configureMedia();
 }
 
 void KDIconView::lineupIcons(QIconView::Arrangement align)
@@ -500,10 +500,8 @@ void KDIconView::lineupIcons(QIconView::Arrangement align)
     else
         rearrangeIcons();
 
-    KConfig * config = KGlobal::config();
-    config->setGroup( "Desktop Icons" );
-    config->writeEntry( "VertAlign", m_bVertAlign );
-    config->sync();
+    KDesktopSettings::setVertAlign( m_bVertAlign );
+    KDesktopSettings::writeConfig();
 }
 
 QStringList KDIconView::selectedURLs()
@@ -595,7 +593,7 @@ void KDIconView::wheelEvent( QWheelEvent* e )
       emit wheelRolled( e->delta() );
       return;
     }
-    
+
     KonqIconViewWidget::wheelEvent( e );
 }
 
@@ -707,27 +705,31 @@ bool KDIconView::deleteGlobalDesktopFiles()
             itemsLeft = true;
             continue; // File is in users own Desktop directory
         }
-        
+
         if (!isDesktopFile(fItem))
         {
             itemsLeft = true;
             continue; // Not a .desktop file
         }
-        
+
         KDesktopFile df(desktopPath + fItem->url().fileName());
         df.writeEntry("Hidden", true);
         df.sync();
-        
+
         delete it;
     }
     return !itemsLeft;
 }
 
-void KDIconView::slotTrash()
+void KDIconView::slotTrashActivated( KAction::ActivationReason reason, Qt::ButtonState state )
 {
     if (deleteGlobalDesktopFiles())
        return; // All items deleted
-    KonqOperations::del(this, KonqOperations::TRASH, selectedUrls());
+
+    if ( reason == KAction::PopupMenuActivation && ( state & Qt::ShiftButton ) )
+       KonqOperations::del(this, KonqOperations::DEL, selectedUrls());
+    else
+       KonqOperations::del(this, KonqOperations::TRASH, selectedUrls());
 }
 
 void KDIconView::slotDelete()
@@ -741,7 +743,7 @@ void KDIconView::slotDelete()
 
 // This method is called when right-clicking over one or more items
 // Not to be confused with the global popup-menu, KRootWm, when doing RMB on the desktop
-void KDIconView::popupMenu( const QPoint &_global, KFileItemList _items )
+void KDIconView::popupMenu( const QPoint &_global, const KFileItemList& _items )
 {
     if (!kapp->authorize("action/kdesktop_rmb")) return;
     if (!m_dirLister) return;
@@ -752,13 +754,22 @@ void KDIconView::popupMenu( const QPoint &_global, KFileItemList _items )
     if (pasteTo)
         pasteTo->setEnabled( m_actionCollection.action( "paste" )->isEnabled() );
 
+    bool hasMediaFiles = false;
+    KFileItemListIterator it(_items);
+    for (; it.current() && !hasMediaFiles; ++it) {
+        hasMediaFiles = it.current()->url().protocol() == "media";
+    }
+
+    KParts::BrowserExtension::PopupFlags itemFlags = KParts::BrowserExtension::DefaultPopupItems;
+    if ( hasMediaFiles )
+        itemFlags |= KParts::BrowserExtension::NoDeletion;
     KonqPopupMenu * popupMenu = new KonqPopupMenu( KonqBookmarkManager::self(), _items,
                                                    url(),
                                                    m_actionCollection,
                                                    KRootWm::self()->newMenu(),
                                                    this,
                                                    KonqPopupMenu::ShowProperties | KonqPopupMenu::ShowNewWindow,
-                                                   KParts::BrowserExtension::DefaultPopupItems );
+                                                   itemFlags );
 
     popupMenu->exec( _global );
     delete popupMenu;
@@ -846,7 +857,7 @@ bool KDIconView::makeFriendlyText( KFileIVI *fileIVI )
         cfg.setDesktopGroup();
         if (cfg.readBoolEntry("Hidden"))
             return false;
-            
+
         if (cfg.readBoolEntry( "NoDisplay", false ))
             return false;
 
@@ -883,6 +894,8 @@ void KDIconView::slotClear()
 
 void KDIconView::slotNewItems( const KFileItemList & entries )
 {
+  bool firstRun = (count() == 0);  // no icons yet, this seems to be the initial loading
+
   // delay updates until all new items have been created
   setUpdatesEnabled( false );
   QRect area = iconArea();
@@ -905,7 +918,7 @@ void KDIconView::slotNewItems( const KFileItemList & entries )
       QString fileName = url.fileName();
       if (QFile::exists(desktopPath + fileName))
          continue; // Don't duplicate entry
-     
+
       QString mostLocal = locate("appdata", "Desktop/"+fileName);
       if (!mostLocal.isEmpty() && (mostLocal != url.path()))
          continue; // Don't duplicate entry
@@ -953,14 +966,14 @@ void KDIconView::slotNewItems( const KFileItemList & entries )
 
         QRect oldPos = fileIVI->rect();
         fileIVI->move( x, y );
-        if ( !isFreePosition( fileIVI ) ) // if we can't put it there, then let QIconView decide
+        if ( !firstRun && !isFreePosition( fileIVI ) ) // if we can't put it there, then let QIconView decide
         {
             kdDebug(1214)<<"slotNewItems() pos was not free :-("<<endl;
             fileIVI->move( oldPos.x(), oldPos.y() );
             m_dotDirectory->deleteGroup( group );
             m_bNeedSave = true;
         }
-        else 
+        else
         {
             kdDebug(1214)<<"Using saved position"<<endl;
         }
@@ -976,11 +989,13 @@ void KDIconView::slotNewItems( const KFileItemList & entries )
       }
     }
   }
+
+  setIconArea( area );
+
   // align on grid
   if ( m_autoAlign )
       lineupIcons();
 
-  setIconArea( area );
   setUpdatesEnabled( true );
 }
 
@@ -989,7 +1004,7 @@ void KDIconView::slotNewItems( const KFileItemList & entries )
 // see also KonqKfmIconView::slotRefreshItems
 void KDIconView::slotRefreshItems( const KFileItemList & entries )
 {
-    //kdDebug(1204) << "KDIconView::slotRefreshItems" << endl;
+    kdDebug(1204) << "KDIconView::slotRefreshItems" << endl;
     bool bNeedPreviewJob = false;
     KFileItemListIterator rit(entries);
     for (; rit.current(); ++rit)
@@ -1043,17 +1058,51 @@ void KDIconView::refreshIcons()
     for ( ; it ; it = it->nextItem() )
     {
         KFileIVI * fileIVI = static_cast<KFileIVI *>(it);
+        fileIVI->item()->refresh();
         fileIVI->refreshIcon( true );
         makeFriendlyText( fileIVI );
     }
 }
 
 
+void KDIconView::FilesAdded( const KURL & directory )
+{
+    if ( directory.path().length() <= 1 && directory.protocol() == "trash" )
+        refreshTrashIcon();
+}
+
+void KDIconView::FilesRemoved( const KURL::List & fileList )
+{
+    if ( !fileList.isEmpty() ) {
+        const KURL url = fileList.first();
+        if ( url.protocol() == "trash" )
+            refreshTrashIcon();
+    }
+}
+
+void KDIconView::refreshTrashIcon()
+{
+    QIconViewItem *it = firstItem();
+    for ( ; it ; it = it->nextItem() )
+    {
+        KFileIVI * fileIVI = static_cast<KFileIVI *>(it);
+        KFileItem* item = fileIVI->item();
+        if ( isDesktopFile( item ) ) {
+            KSimpleConfig cfg( item->url().path(), true );
+            cfg.setDesktopGroup();
+            if ( cfg.readEntry( "Type" ) == "Link" &&
+                 cfg.readEntry( "URL" ) == "trash:/" ) {
+                fileIVI->refreshIcon( true );
+            }
+        }
+    }
+}
+
 // -----------------------------------------------------------------------------
 
 void KDIconView::slotDeleteItem( KFileItem * _fileitem )
 {
-//    kdDebug(1204) << "KDIconView::slotDeleteItems" << endl;
+    kdDebug(1204) << "KDIconView::slotDeleteItems" << endl;
     // we need to find out the KFileIVI containing the fileitem
     QIconViewItem *it = firstItem();
     while ( it ) {
@@ -1280,8 +1329,11 @@ void KDIconView::contentsDropEvent( QDropEvent * e )
         setLastIconPosition( e->pos() );
         KonqIconViewWidget::contentsDropEvent( e );
     }
-    if (QIconDrag::canDecode(e))
+    if (QIconDrag::canDecode(e)) {
       emit iconMoved();
+      if ( !m_autoAlign )    // if autoAlign, positions were saved in lineupIcons
+        saveIconPositions();
+    }
 }
 
 // don't scroll when someone uses his nifty mouse wheel
@@ -1313,10 +1365,10 @@ void KDIconView::updateWorkArea( const QRect &wr )
         for ( item = firstItem(); item; item = item->nextItem() ) {
             QRect r( item->rect() );
             int dx = 0, dy = 0;
-            if ( r.bottom() > visibleHeight() )
-                dy = visibleHeight() - r.bottom() - 1;
-            if ( r.right() > visibleWidth() )
-                dx = visibleWidth() - r.right() - 1;
+            if ( r.bottom() > wr.bottom() )
+                dy = wr.bottom() - r.bottom() - 1;
+            if ( r.right() > wr.right() )
+                dx = wr.right() - r.right() - 1;
             if ( dx != 0 || dy != 0 ) {
                 needRepaint = true;
                 item->moveBy( dx, dy );
@@ -1483,6 +1535,9 @@ void KDIconView::saveIconPositions()
 {
   kdDebug(1214) << "KDIconView::saveIconPositions" << endl;
 
+  if (!m_bEditableDesktopIcons)
+    return; // Don't save position
+
   QString prefix = iconPositionGroupPrefix();
   QIconViewItem *it = firstItem();
   if ( !it )
@@ -1498,31 +1553,10 @@ void KDIconView::saveIconPositions()
     m_dotDirectory->setGroup( prefix + item->url().fileName() );
     kdDebug(1214) << "KDIconView::saveIconPositions " << item->url().fileName() << " " << it->x() << " " << it->y() << endl;
     saveIconPosition(m_dotDirectory, it->x(), it->y(), iconArea());
-    m_dotDirectory->writeEntry( "Exists", true );
 
     it = it->nextItem();
   }
 
-  QStringList groups = m_dotDirectory->groupList();
-  QStringList::ConstIterator gIt = groups.begin();
-  QStringList::ConstIterator gEnd = groups.end();
-  for (; gIt != gEnd; ++gIt )
-  {
-    if ( (*gIt).startsWith(prefix) )
-    {
-      m_dotDirectory->setGroup( *gIt );
-      if ( m_dotDirectory->hasKey( "Exists" ) )
-      {
-        m_dotDirectory->deleteEntry( "Exists", false );
-      }
-      else
-      {
-        kdDebug(1214) << "KDIconView::saveIconPositions deleting group " << *gIt << endl;
-        m_dotDirectory->deleteGroup( *gIt );
-      }
-    }
-  }
-
   m_dotDirectory->sync();
 }
 
