@@ -66,6 +66,7 @@
 
 #include "minicli.moc"
 #include "minicli_ui.h"
+#include "kdesktopsettings.h"
 
 #define KDESU_ERR strerror(errno)
 
@@ -73,7 +74,7 @@ Minicli::Minicli( QWidget *parent, const char *name)
         :KDialog( parent, name ),
          m_autoCheckedRunInTerm(false)
 {
-  setCaption( i18n("Run Command") );
+  setPlainCaption( i18n("Run Command") );
   KWin::setIcons( winId(), DesktopIcon("run"), SmallIcon("run") );
 
   QVBoxLayout* mainLayout = new QVBoxLayout( this, 0, KDialog::spacingHint() );
@@ -81,7 +82,7 @@ Minicli::Minicli( QWidget *parent, const char *name)
   mainLayout->addWidget(m_dlg);
 
   m_dlg->lbRunIcon->setPixmap(DesktopIcon("kmenu"));
-  m_dlg->lbComment->setAlignment( Qt::WordBreak );  
+  m_dlg->lbComment->setAlignment( Qt::WordBreak );
 
   m_dlg->cbCommand->setDuplicatesEnabled( false );
   m_dlg->cbCommand->setTrapReturnKey( true );
@@ -149,6 +150,16 @@ Minicli::~Minicli()
   delete m_filterData;
 }
 
+void Minicli::setCommand(const QString& command)
+{
+  m_dlg->cbCommand->lineEdit()->setText(command);
+  m_dlg->cbCommand->lineEdit()->deselect();
+  int firstSpace = command.find(' ');
+  if (firstSpace > 0) {
+    m_dlg->cbCommand->lineEdit()->setSelection(firstSpace+1, command.length());
+  }
+}
+
 void Minicli::setMaxCommandBoxWidth()
 {
   int maxWidth = qApp->desktop()->screenGeometry(this).width();
@@ -170,6 +181,8 @@ void Minicli::show()
   setMaxCommandBoxWidth();
   KWin::setState( winId(), NET::StaysOnTop );
   setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
+  adjustSize();
+  setFixedSize(size());
   KDialog::show();
   adjustSize();
   setFixedSize(size());
@@ -177,12 +190,9 @@ void Minicli::show()
 
 void Minicli::loadConfig()
 {
-  KConfig *config = KGlobal::config();
-  config->setGroup("MiniCli");
-
-  QStringList histList = config->readPathListEntry("History");
-  int maxHistory = config->readNumEntry("HistoryLength", 50);
-  m_terminalAppList = config->readPathListEntry("TerminalApps");
+  QStringList histList = KDesktopSettings::history();
+  int maxHistory = KDesktopSettings::historyLength();
+  m_terminalAppList = KDesktopSettings::terminalApps();
 
   if (m_terminalAppList.isEmpty())
     m_terminalAppList << "ls"; // Default
@@ -193,13 +203,13 @@ void Minicli::loadConfig()
   m_dlg->cbCommand->setHistoryItems( histList );
   m_dlg->cbCommand->blockSignals( block );
 
-  QStringList compList = config->readPathListEntry("CompletionItems");
+  QStringList compList = KDesktopSettings::completionItems();
   if( compList.isEmpty() )
     m_dlg->cbCommand->completionObject()->setItems( histList );
   else
     m_dlg->cbCommand->completionObject()->setItems( compList );
 
-  int mode = config->readNumEntry( "CompletionMode", KGlobalSettings::completionMode() );
+  int mode = KDesktopSettings::completionMode();
   m_dlg->cbCommand->setCompletionMode( (KGlobalSettings::Completion) mode );
 
   m_finalFilters = KURIFilter::self()->pluginNames();
@@ -209,7 +219,7 @@ void Minicli::loadConfig()
   m_middleFilters.remove("localdomainurifilter");
 
   // Provide username completions. Use saner and configurable maximum values.
-  int maxEntries = config->readNumEntry("MaxUsernameCompletions", 30);
+  int maxEntries = KDesktopSettings::maxUsernameCompletions();
   QStringList users;
 
   struct passwd *pw;
@@ -229,13 +239,11 @@ void Minicli::loadConfig()
 
 void Minicli::saveConfig()
 {
-  KConfig *config = KGlobal::config();
-  config->setGroup("MiniCli");
-  config->writePathEntry( "History", m_dlg->cbCommand->historyItems() );
-  config->writePathEntry( "TerminalApps", m_terminalAppList );
-  config->writePathEntry( "CompletionItems", m_dlg->cbCommand->completionObject()->items() );
-  config->writeEntry( "CompletionMode", (int) m_dlg->cbCommand->completionMode() );
-  config->sync();
+  KDesktopSettings::setHistory( m_dlg->cbCommand->historyItems() );
+  KDesktopSettings::setTerminalApps( m_terminalAppList );
+  KDesktopSettings::setCompletionItems( m_dlg->cbCommand->completionObject()->items() );
+  KDesktopSettings::setCompletionMode( m_dlg->cbCommand->completionMode() );
+  KDesktopSettings::writeConfig();
 }
 
 void Minicli::clearHistory()
@@ -329,9 +337,7 @@ void Minicli::keyPressEvent( QKeyEvent* e )
 
 QString Minicli::terminalCommand (const QString& cmd, const QString& args)
 {
-  KConfigGroupSaver gs (KGlobal::config(), "General");
-  QString terminal = KGlobal::config()->readPathEntry("TerminalApplication",
-                                                  "konsole").stripWhiteSpace();
+  QString terminal = KDesktopSettings::terminalApplication().stripWhiteSpace();
   if (terminal.endsWith("konsole"))
     terminal += " --noclose";
 
@@ -358,10 +364,11 @@ int Minicli::runCommand()
     return 1;
 
   QString cmd;
-  if ( m_filterData->uri().isLocalFile() && !m_filterData->uri().hasRef() )
-    cmd = m_filterData->uri().path();
+  KURL uri = m_filterData->uri();
+  if ( uri.isLocalFile() && !uri.hasRef() && uri.query().isEmpty() )
+    cmd = uri.path();
   else
-    cmd = m_filterData->uri().url();
+    cmd = uri.url();
 
   // Determine whether the application should be run through
   // the command line (terminal) interface...
@@ -392,7 +399,7 @@ int Minicli::runCommand()
       pw = getpwuid(getuid());
       if (pw == 0L)
       {
-          KMessageBox::error( this, i18n("You don't exist!\n"));
+          KMessageBox::error( this, i18n("You do not exist.\n"));
         return 1;
       }
     }
@@ -417,7 +424,7 @@ int Minicli::runCommand()
 
       if (proc_checkpwd.checkInstall(m_dlg->lePassword->password()) != 0)
       {
-          KMessageBox::sorry(this, i18n("Incorrect password! Please try again."));
+          KMessageBox::sorry(this, i18n("Incorrect password; please try again."));
         return 1;
       }
     }
@@ -505,7 +512,7 @@ int Minicli::runCommand()
           {
             // Look for desktop file
             KService::Ptr service = KService::serviceByDesktopName(cmd);
-            if (service)
+            if (service && service->isValid() && service->type() == "Application")
             {
               KRun::run(*service, KURL::List());
               return 0;
@@ -521,17 +528,14 @@ int Minicli::runCommand()
             if( m_filterData->hasArgsAndOptions() )
               cmd += m_filterData->argsAndOptions();
 
-            //kdDebug(1207) << "Command to execute: " << cmd << endl;
-            //kdDebug(1207) << "Executable name: " << exec << endl;
-
             break;
           }
           else
           {
-              KMessageBox::sorry( this, i18n("<center><b>%1</b></center>\n"
-                                        "You do not have permission to execute "
-                                        "this command!")
-                                        .arg( QStyleSheet::convertFromPlainText(cmd) ));
+            KMessageBox::sorry( this, i18n("<center><b>%1</b></center>\n"
+                                      "You do not have permission to execute "
+                                      "this command.")
+                                      .arg( QStyleSheet::convertFromPlainText(cmd) ));
             return 1;
           }
         }
@@ -541,21 +545,22 @@ int Minicli::runCommand()
         {
           // Look for desktop file
           KService::Ptr service = KService::serviceByDesktopName(cmd);
-          if (service)
-          {
-            KRun::run(*service, KURL::List());
-            return 0;
-          }
-          service = KService::serviceByName(cmd);
-          if (service)
+          if (service && service->isValid() && service->type() == "Application")
           {
             KRun::run(*service, KURL::List());
             return 0;
           }
 
-            KMessageBox::sorry( this, i18n("<center><b>%1</b></center>\n"
-                                      "Could not run the specified command!")
-                                      .arg( QStyleSheet::convertFromPlainText(cmd) ));
+          service = KService::serviceByName(cmd);
+          if (service && service->isValid() && service->type() == "Application")
+          {
+            KRun::run(*service, KURL::List());
+            return 0;
+          }
+
+          KMessageBox::sorry( this, i18n("<center><b>%1</b></center>\n"
+                                    "Could not run the specified command.")
+                                    .arg( QStyleSheet::convertFromPlainText(cmd) ));
           return 1;
         }
       }
@@ -565,8 +570,8 @@ int Minicli::runCommand()
       return 0;
     else
     {
-        KMessageBox::sorry( this, i18n("<center><b>%1</b></center>\n"
-                                  "The specified command does not exist!").arg(cmd) );
+      KMessageBox::sorry( this, i18n("<center><b>%1</b></center>\n"
+                                "The specified command does not exist.").arg(cmd) );
       return 1; // Let the user try again...
     }
   }
@@ -667,7 +672,7 @@ void Minicli::parseLine( bool final )
 
 void Minicli::setIcon ()
 {
-  if( m_iconName.isEmpty() || m_iconName == "unknown" )
+  if( m_iconName.isEmpty() || m_iconName == "unknown" || m_iconName == "kde" )
     m_iconName = QString::fromLatin1("kmenu");
 
   QPixmap icon = DesktopIcon( m_iconName );
@@ -763,13 +768,10 @@ void Minicli::slotTerminal(bool enable)
     m_iconName = QString::fromLatin1( "konsole" );
     setIcon();
   }
-  else
+  else if (!m_prevIconName.isEmpty())
   {
-    if (!m_prevIconName.isEmpty())
-    {
       m_iconName = m_prevIconName;
       setIcon();
-    }
   }
 }
 
@@ -813,7 +815,7 @@ void Minicli::slotRealtime(bool enabled)
                 i18n("Running a realtime application can be very dangerous. "
                     "If the application misbehaves, the system might hang "
                     "unrecoverably.\nAre you sure you want to continue?"),
-                i18n("Warning - Run Command"))
+                i18n("Warning - Run Command"), KGuiItem(i18n("&Run Realtime")),QString::null,KMessageBox::Notify|KMessageBox::PlainCaption)
         != KMessageBox::Continue )
     {
       m_iScheduler = StubProcess::SchedNormal;
@@ -841,7 +843,7 @@ void Minicli::slotPriority(int priority)
 QString Minicli::calculate(const QString &exp)
 {
    QString result;
-   QString cmd = QString("echo $[%1]").arg(exp);
+   QString cmd = QString("echo $((%1))").arg(exp);
    FILE *fs = popen(QFile::encodeName(cmd).data(), "r");
    if (fs)
    {
