@@ -21,21 +21,20 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ******************************************************************/
 
-#include <qlayout.h>
-
-#include <kdebug.h>
-#include <kconfig.h>
-#include <kglobal.h>
-#include <kpanelapplet.h>
-#include <kapplication.h>
 #include <algorithm>
 
+#include <qlayout.h>
+
+#include <kapplication.h>
+#include <kconfig.h>
+#include <kdebug.h>
+#include <kglobal.h>
+#include <kmultipledrag.h>
+#include <kpanelapplet.h>
+#include <kurldrag.h>
+
 #include "global.h"
-#include "panel.h"
-// TODO: Martijn Klingens
-// #include "buttonop_mnu.h"
 #include "appletop_mnu.h"
-#include "popupposition.h"
 
 #include "panelbutton.h"
 #include "bookmarksbutton.h"
@@ -43,7 +42,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "desktopbutton.h"
 #include "extensionbutton.h"
 #include "kbutton.h"
+#include "kicker.h"
+#include "kickertip.h"
 #include "nonkdeappbutton.h"
+#include "paneldrag.h"
 #include "servicebutton.h"
 #include "servicemenubutton.h"
 #include "urlbutton.h"
@@ -68,21 +70,25 @@ bool ButtonContainer::isValid() const
    return true; // Can this happen?
 }
 
-
 // Buttons Shouldn't be square when larger than a certain size.
 int ButtonContainer::widthForHeight(int height) const
-{   if (isValid()) {
-        int val=_button->preferredDimension(height);
+{
+    if (isValid())
+    {
+        int val = _button->preferredDimension(height);
         //kdDebug() << "ButtonContainer::widthForHeight("<<height<<"):" << val << endl<<flush;
         return val;
     }
+
     //kdDebug() << "ButtonContainer::widthForHeight -  NO VALID BUTTON!"<< endl<<flush;
     return height;
 }
 
 int ButtonContainer::heightForWidth(int width)  const
-{   if (isValid()) {
-        int val=_button->preferredDimension(width);
+{
+    if (isValid())
+    {
+        int val = _button->preferredDimension(width);
         //kdDebug() << "ButtonContainer::heightForWidth("<<width<<"):" << val << endl<<flush;
         return val;
     }
@@ -90,31 +96,48 @@ int ButtonContainer::heightForWidth(int width)  const
     return width;
 }
 
+void ButtonContainer::setBackground()
+{
+    PanelButton* b = button();
+    if (b)
+    {
+        b->repaint();
+    }
+}
 
 void ButtonContainer::configure()
 {
-    if(_button) _button->configure();
+    if (_button)
+    {
+        _button->configure();
+    }
 }
 
-void ButtonContainer::doSaveConfiguration( KConfigGroup& config, bool layoutOnly) const
+void ButtonContainer::doSaveConfiguration(KConfigGroup& config, bool layoutOnly) const
 {
-    if(_button && !layoutOnly) _button->saveConfig( config );
+    // immutability is checked by ContainerBase
+    if (_button && !layoutOnly)
+    {
+        _button->saveConfig(config);
+    }
 }
 
-void ButtonContainer::slotSetPopupDirection(Direction d)
+void ButtonContainer::setPopupDirection(KPanelApplet::Direction d)
 {
-    BaseContainer::slotSetPopupDirection(d);
+    BaseContainer::setPopupDirection(d);
+
+    if (_button)
+    {
+        _button->setPopupDirection(d);
+    }
+}
+
+void ButtonContainer::setOrientation(Orientation o)
+{
+    BaseContainer::setOrientation(o);
 
     if(_button)
-        _button->slotSetPopupDirection(d);
-}
-
-void ButtonContainer::slotSetOrientation(Orientation o)
-{
-    BaseContainer::slotSetOrientation(o);
-
-    if(_button)
-        _button->slotSetOrientation(o);
+        _button->setOrientation(o);
 }
 
 void ButtonContainer::embedButton(PanelButton* b)
@@ -128,16 +151,52 @@ void ButtonContainer::embedButton(PanelButton* b)
     _button->installEventFilter(this);
     _layout->add(_button);
     connect(_button, SIGNAL(requestSave()), SIGNAL(requestSave()));
+    connect(_button, SIGNAL(removeme()), SLOT(removeRequested()));
+    connect(_button, SIGNAL(dragme(const QPixmap)),
+            SLOT(dragButton(const QPixmap)));
+    connect(_button, SIGNAL(dragme(const KURL::List, const QPixmap)),
+            SLOT(dragButton(const KURL::List, const QPixmap)));
 }
 
-QPopupMenu* ButtonContainer::createOpMenu() const
+QPopupMenu* ButtonContainer::createOpMenu()
 {
-    // TODO: Martijn Klingens
-    // _opMnu = new PanelButtonOpMenu(_actions, _button->title(),
-    //                                _button->icon());
-
     return new PanelAppletOpMenu(_actions, appletOpMenu(), 0, _button->title(),
-                                 _button->icon(), const_cast<ButtonContainer*>(this));
+                                 _button->icon(), this);
+}
+
+void ButtonContainer::removeRequested()
+{
+    if (isImmutable())
+    {
+        return;
+    }
+
+    emit removeme(this);
+}
+
+void ButtonContainer::dragButton(const KURL::List urls, const QPixmap icon)
+{
+    if (isImmutable())
+    {
+        return;
+    }
+
+    KMultipleDrag* dd = new KMultipleDrag(this);
+    dd->addDragObject(new KURLDrag(urls, 0));
+    dd->addDragObject(new PanelDrag(this, 0));
+    dd->setPixmap(icon);
+    grabKeyboard();
+    dd->dragMove();
+    releaseKeyboard();
+}
+
+void ButtonContainer::dragButton(const QPixmap icon)
+{
+    PanelDrag* dd = new PanelDrag(this, this);
+    dd->setPixmap(icon);
+    grabKeyboard();
+    dd->drag();
+    releaseKeyboard();
 }
 
 bool ButtonContainer::eventFilter(QObject *o, QEvent *e)
@@ -157,6 +216,11 @@ bool ButtonContainer::eventFilter(QObject *o, QEvent *e)
         {
         case MidButton:
         {
+            if (m_immutable)
+            {
+                break;
+            }
+
             _button->setDown(true);
             _moveOffset = me->pos();
             emit moveme(this);
@@ -166,16 +230,20 @@ bool ButtonContainer::eventFilter(QObject *o, QEvent *e)
 
         case RightButton:
         {
-            if (!kapp->authorizeKAction("kicker_rmb"))
+            if (!kapp->authorizeKAction("kicker_rmb") ||
+                m_immutable)
+            {
                 break;
+            }
 
-            _button->setZoomEnabled(false);
-            qApp->syncX();
-            qApp->processEvents();
+            KickerTip::enableTipping(false);
 
             QPopupMenu* menu = opMenu();
             connect( menu, SIGNAL( aboutToHide() ), this, SLOT( slotMenuClosed() ) );
-            switch(menu->exec( popupPosition( popupDirection(), menu, this, me->pos() )))
+            QPoint pos = KickerLib::popupPosition(popupDirection(), menu, this,
+                                                 (orientation() == Horizontal) ?
+                                                    QPoint(0, 0) : me->pos());
+            switch (menu->exec(pos))
             {
             case PanelAppletOpMenu::Move:
                 _moveOffset = rect().center();
@@ -198,7 +266,7 @@ bool ButtonContainer::eventFilter(QObject *o, QEvent *e)
                 break;
             }
 
-            _button->setZoomEnabled(true);
+            KickerTip::enableTipping(true);
             sentinal = false;
             return true;
         }
@@ -214,8 +282,10 @@ bool ButtonContainer::eventFilter(QObject *o, QEvent *e)
 
 void ButtonContainer::completeMoveOperation()
 {
-    if(_button)
+    if (_button)
+    {
         _button->setDown(false);
+    }
 }
 
 void ButtonContainer::slotMenuClosed()
@@ -224,7 +294,23 @@ void ButtonContainer::slotMenuClosed()
         _button->setDown(false);
 }
 
+void ButtonContainer::checkImmutability(const KConfigGroup& config)
+{
+    m_immutable = Kicker::the()->isImmutable() ||
+                  config.groupIsImmutable() ||
+                  config.entryIsImmutable("ConfigFile") ||
+                  config.entryIsImmutable("FreeSpace2");
+}
+
 // KMenuButton containerpan
+KMenuButtonContainer::KMenuButtonContainer(const KConfigGroup& config, QPopupMenu *opMenu, QWidget* parent)
+  : ButtonContainer(opMenu, parent)
+{
+    checkImmutability(config);
+    embedButton( new KButton(this) );
+    _actions = PanelAppletOpMenu::KMenuEditor;
+}
+
 KMenuButtonContainer::KMenuButtonContainer(QPopupMenu *opMenu, QWidget* parent)
   : ButtonContainer(opMenu, parent)
 {
@@ -255,6 +341,14 @@ DesktopButtonContainer::DesktopButtonContainer(QPopupMenu *opMenu, QWidget* pare
     embedButton( new DesktopButton(this) );
 }
 
+DesktopButtonContainer::DesktopButtonContainer(const KConfigGroup& config,
+                                               QPopupMenu *opMenu, QWidget* parent)
+  : ButtonContainer(opMenu, parent)
+{
+    checkImmutability(config);
+    embedButton( new DesktopButton(this) );
+}
+
 // ServiceButton container
 ServiceButtonContainer::ServiceButtonContainer( const QString& desktopFile,
                                                 QPopupMenu* opMenu, QWidget* parent)
@@ -276,10 +370,21 @@ ServiceButtonContainer::ServiceButtonContainer( const KConfigGroup& config,
                                                 QPopupMenu* opMenu, QWidget* parent)
   : ButtonContainer(opMenu, parent)
 {
+    checkImmutability(config);
     embedButton( new ServiceButton( config, this ) );
     _actions = KPanelApplet::Preferences;
 }
 
+QString ServiceButtonContainer::icon() const
+{
+    return button()->icon();
+}
+
+QString ServiceButtonContainer::visibleName() const
+{
+    return button()->title();
+}
+
 // URLButton container
 URLButtonContainer::URLButtonContainer( const QString& url, QPopupMenu* opMenu, QWidget* parent )
   : ButtonContainer(opMenu, parent)
@@ -291,10 +396,21 @@ URLButtonContainer::URLButtonContainer( const QString& url, QPopupMenu* opMenu,
 URLButtonContainer::URLButtonContainer( const KConfigGroup& config, QPopupMenu* opMenu, QWidget* parent)
   : ButtonContainer(opMenu, parent)
 {
+    checkImmutability(config);
     embedButton( new URLButton( config, this ) );
     _actions = KPanelApplet::Preferences;
 }
 
+QString URLButtonContainer::icon() const
+{
+    return button()->icon();
+}
+
+QString URLButtonContainer::visibleName() const
+{
+    return button()->title();
+}
+
 // BrowserButton container
 BrowserButtonContainer::BrowserButtonContainer(const QString &startDir, QPopupMenu* opMenu, const QString& icon, QWidget* parent)
   : ButtonContainer(opMenu, parent)
@@ -306,6 +422,7 @@ BrowserButtonContainer::BrowserButtonContainer(const QString &startDir, QPopupMe
 BrowserButtonContainer::BrowserButtonContainer( const KConfigGroup& config, QPopupMenu* opMenu, QWidget* parent)
   : ButtonContainer(opMenu, parent)
 {
+    checkImmutability(config);
     embedButton( new BrowserButton(config, this) );
     _actions = KPanelApplet::Preferences;
 }
@@ -320,10 +437,28 @@ ServiceMenuButtonContainer::ServiceMenuButtonContainer(const QString& relPath, Q
 ServiceMenuButtonContainer::ServiceMenuButtonContainer( const KConfigGroup& config, QPopupMenu* opMenu, QWidget* parent)
   : ButtonContainer(opMenu, parent)
 {
+    checkImmutability(config);
     embedButton( new ServiceMenuButton(config, this) );
 }
 
+QString ServiceMenuButtonContainer::icon() const
+{
+    return button()->icon();
+}
+
+QString ServiceMenuButtonContainer::visibleName() const
+{
+    return button()->title();
+}
+
 // WindowListButton container
+WindowListButtonContainer::WindowListButtonContainer(const KConfigGroup& config, QPopupMenu *opMenu, QWidget* parent)
+  : ButtonContainer(opMenu, parent)
+{
+    checkImmutability(config);
+    embedButton( new WindowListButton(this) );
+}
+
 WindowListButtonContainer::WindowListButtonContainer(QPopupMenu *opMenu, QWidget* parent)
   : ButtonContainer(opMenu, parent)
 {
@@ -331,25 +466,37 @@ WindowListButtonContainer::WindowListButtonContainer(QPopupMenu *opMenu, QWidget
 }
 
 // BookmarkButton container
+BookmarksButtonContainer::BookmarksButtonContainer(const KConfigGroup& config, QPopupMenu *opMenu, QWidget* parent)
+  : ButtonContainer(opMenu, parent)
+{
+    checkImmutability(config);
+    embedButton( new BookmarksButton(this) );
+    _actions = PanelAppletOpMenu::BookmarkEditor;
+}
+
 BookmarksButtonContainer::BookmarksButtonContainer(QPopupMenu *opMenu, QWidget* parent)
   : ButtonContainer(opMenu, parent)
 {
     embedButton( new BookmarksButton(this) );
+    _actions = PanelAppletOpMenu::BookmarkEditor;
 }
 
+// NonKDEAppButton container
 NonKDEAppButtonContainer::NonKDEAppButtonContainer(
+        const QString & name,
         const QString &filePath, const QString &icon,
-	const QString &cmdLine, bool inTerm,
-	QPopupMenu* opMenu, QWidget* parent)
+        const QString &cmdLine, bool inTerm,
+        QPopupMenu* opMenu, QWidget* parent)
   : ButtonContainer(opMenu, parent)
 {
-    embedButton( new NonKDEAppButton(filePath, icon, cmdLine, inTerm, this) );
+    embedButton( new NonKDEAppButton(name, filePath, icon, cmdLine, inTerm, this) );
     _actions = KPanelApplet::Preferences;
 }
 
 NonKDEAppButtonContainer::NonKDEAppButtonContainer( const KConfigGroup& config, QPopupMenu* opMenu, QWidget *parent)
   : ButtonContainer(opMenu, parent)
 {
+    checkImmutability(config);
     embedButton( new NonKDEAppButton(config, this) );
     _actions = KPanelApplet::Preferences;
 }
@@ -364,5 +511,17 @@ ExtensionButtonContainer::ExtensionButtonContainer(const QString& df, QPopupMenu
 ExtensionButtonContainer::ExtensionButtonContainer( const KConfigGroup& config, QPopupMenu* opMenu, QWidget *parent)
   : ButtonContainer(opMenu, parent)
 {
+    checkImmutability(config);
     embedButton( new ExtensionButton(config, this) );
 }
+
+QString ExtensionButtonContainer::icon() const
+{
+    return button()->icon();
+}
+
+QString ExtensionButtonContainer::visibleName() const
+{
+    return button()->title();
+}
+
