@@ -25,37 +25,21 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef __taskcontainer_h__
 #define __taskcontainer_h__
 
-#include <qptrlist.h>
-#include <qpixmap.h>
-#include <qimage.h>
 #include <qtimer.h>
 #include <qtoolbutton.h>
 
-#include "taskmanager.h"
 #include "taskbar.h"
 
-class LMBMenu;
-class RMBMenu;
-class TaskContainer;
-
-typedef QPtrList<QPixmap> PixmapList;
-typedef QPtrList<TaskContainer> TaskContainerList;
-typedef QPtrListIterator<TaskContainer> TaskContainerListIterator;
-
 class TaskContainer : public QToolButton
 {
     Q_OBJECT
 
 public:
-    TaskContainer( Task*, TaskManager*, bool show, bool sort, bool icon, bool iconifiedOnly, QWidget *parent = 0, const char *name = 0 );
-    TaskContainer( Startup*, PixmapList*, TaskManager*, bool show, bool sort, bool icon, bool iconifiedOnly, QWidget *parent = 0, const char *name = 0 );
+    TaskContainer( Task*, TaskBar*, QWidget *parent = 0, const char *name = 0 );
+    TaskContainer( Startup*, PixmapList*, TaskBar*, QWidget *parent = 0, const char *name = 0 );
     virtual ~TaskContainer();
 
     void setArrowType( Qt::ArrowType at );
-    void setShowAll( bool );
-    void setSortByDesktop( bool );
-    void setShowIcon( bool );
-    void setShowIconifiedOnly( bool s );
 
     void init();
 
@@ -72,6 +56,7 @@ public:
     bool isEmpty();
     bool onCurrentDesktop();
     bool isIconified();
+    bool isOnScreen();
 
     QString id();
     int desktop();
@@ -82,6 +67,7 @@ public:
     void publishIconGeometry( QPoint );
     void desktopChanged( int );
     void windowChanged( WId );
+    void settingsChanged();
     bool eventFilter( QObject *o, QEvent *e );
 
     static void setLeftButtonAction( TaskBar::Action action )
@@ -92,19 +78,10 @@ public:
         { rightButtonAction = action; }
 
     int taskCount() const { return tasks.count(); }
-    int filteredTaskCount() const { return ftasks.count(); }
+    int filteredTaskCount() const { return m_filteredTasks.count(); }
 
     bool activateNextTask( bool forward, bool& forcenext );
 
-protected slots:
-    void animationTimerFired();
-    void attentionTimerFired();
-    void slotClicked();
-    void dragSwitch();
-    void update();
-    void setLastActivated();
-    void taskChanged();
-
 protected:
     void drawButton( QPainter* );
     void resizeEvent( QResizeEvent* );
@@ -112,7 +89,6 @@ protected:
     void mouseReleaseEvent( QMouseEvent* );
     void dragEnterEvent( QDragEnterEvent* );
     void dragLeaveEvent( QDragLeaveEvent* );
-    void wheelEvent( QWheelEvent* );
 
     void performAction( TaskBar::Action action );
     void popupMenu( TaskBar::Action action );
@@ -120,26 +96,32 @@ protected:
     void updateFilteredTaskList();
     QColor blendColors( QColor, QColor );
 
+protected slots:
+    void animationTimerFired();
+    void attentionTimerFired();
+    void slotClicked();
+    void dragSwitch();
+    void update();
+    void iconChanged();
+    void setLastActivated();
+    void taskChanged();
+
 private:
     void checkAttention( const Task* changed_task = NULL );
     QString                     sid;
-    QString                     currentTooltip;
     QTimer                      animationTimer;
     QTimer                      dragSwitchTimer;
     int                         currentFrame;
     QRect                       iconRect;
     QPixmap                     animBg;
     TaskList                    tasks;
-    TaskList                    ftasks;
+    TaskList                    m_filteredTasks;
     Task*                       lastActivated;
+    QPopupMenu*                 m_menu;
     StartupList                 startups;
     PixmapList*                 frames;
     ArrowType                   arrowType;
-    TaskManager*                taskManager;
-    bool                        showAll;
-    bool                        showOnlyIconified;
-    bool                        sortByDesktop;
-    bool                        showIcon;
+    TaskBar*                    taskBar;
     static TaskBar::Action      leftButtonAction;
     static TaskBar::Action      middleButtonAction;
     static TaskBar::Action      rightButtonAction;
