@@ -67,6 +67,7 @@ KonqView::KonqView( KonqViewFactory &viewFactory,
   m_pKonqFrame->setView( this );
 
   m_sLocationBarURL = "";
+  m_pageSecurity = KonqMainWindow::NotCrypted;
   m_bLockHistory = false;
   m_doPost = false;
   m_pMainWindow = mainWindow;
@@ -104,7 +105,7 @@ KonqView::KonqView( KonqViewFactory &viewFactory,
 
 KonqView::~KonqView()
 {
-  kdDebug(1202) << "KonqView::~KonqView : part = " << m_pPart << endl;
+  //kdDebug(1202) << "KonqView::~KonqView : part = " << m_pPart << endl;
 
   if (KonqMainWindow::s_crashlog_file) {
      QString part_url;
@@ -121,6 +122,7 @@ KonqView::~KonqView()
   // We did so ourselves for passive views
   if (m_pPart != 0L)
   {
+    finishedWithCurrentURL();
     if ( isPassiveMode() )
       disconnect( m_pPart, SIGNAL( destroyed() ), m_pMainWindow->viewManager(), SLOT( slotObjectDestroyed() ) );
 
@@ -128,12 +130,13 @@ KonqView::~KonqView()
   }
 
   setRun( 0L );
-  kdDebug(1202) << "KonqView::~KonqView " << this << " done" << endl;
+  //kdDebug(1202) << "KonqView::~KonqView " << this << " done" << endl;
 }
 
-void KonqView::openURL( const KURL &url, const QString & locationBarURL, const QString & nameFilter )
+void KonqView::openURL( const KURL &url, const QString & locationBarURL,
+                        const QString & nameFilter, bool tempFile )
 {
-  kdDebug(1202) << "KonqView::openURL url=" << url.url() << " locationBarURL=" << locationBarURL << endl;
+  kdDebug(1202) << "KonqView::openURL url=" << url << " locationBarURL=" << locationBarURL << endl;
   setServiceTypeInExtension();
 
   if (KonqMainWindow::s_crashlog_file) {
@@ -186,6 +189,7 @@ void KonqView::openURL( const KURL &url, const QString & locationBarURL, const Q
 
   callExtensionStringMethod( "setNameFilter(const QString&)", nameFilter );
   setLocationBarURL( locationBarURL );
+  setPageSecurity(KonqMainWindow::NotCrypted);
 
   if ( !args.reload )
   {
@@ -199,6 +203,15 @@ void KonqView::openURL( const KURL &url, const QString & locationBarURL, const Q
   }
 
   m_bAborted = false;
+  if ( tempFile ) {
+      // Store the path to the tempfile. Yes, we could store a bool only,
+      // but this would be more dangerous. If anything goes wrong in the code,
+      // we might end up deleting a real file.
+      if ( url.isLocalFile() )
+          m_tempFile = url.path();
+      else
+          kdWarning(1202) << "Tempfile option is set, but URL is remote: " << url << endl;
+  }
 
   m_pPart->openURL( url );
 
@@ -345,7 +358,7 @@ bool KonqView::changeViewMode( const QString &serviceType,
     // each time we change the view mode.
     // We don't do it in switchView either because it's called from the constructor too,
     // where the location bar url isn't set yet.
-    kdDebug(1202) << "Giving focus to new part " << m_pPart << endl;
+    //kdDebug(1202) << "Giving focus to new part " << m_pPart << endl;
     m_pMainWindow->viewManager()->setActivePart( m_pPart );
   }
   return true;
@@ -386,6 +399,9 @@ void KonqView::connectPart(  )
       connect( ext, SIGNAL( setIconURL( const KURL & ) ),
                this, SLOT( setIconURL( const KURL & ) ) );
 
+      connect( ext, SIGNAL( setPageSecurity( int ) ),
+               this, SLOT( setPageSecurity( int ) ) );
+
       connect( ext, SIGNAL( createNewWindow( const KURL &, const KParts::URLArgs & ) ),
                m_pMainWindow, SLOT( slotCreateNewWindow( const KURL &, const KParts::URLArgs & ) ) );
 
@@ -513,7 +529,7 @@ void KonqView::slotRequestFocus( KParts::ReadOnlyPart * )
 
 void KonqView::setLoading( bool loading, bool hasPending /*= false*/)
 {
-    kdDebug(1202) << k_funcinfo << "loading=" << loading << " hasPending=" << hasPending << endl;
+    //kdDebug(1202) << "KonqView::setLoading loading=" << loading << " hasPending=" << hasPending << endl;
     m_bLoading = loading;
     m_bPendingRedirection = hasPending;
     if ( m_pMainWindow->currentView() == this )
@@ -544,7 +560,7 @@ void KonqView::slotCompleted()
 
 void KonqView::slotCompleted( bool hasPending )
 {
-  kdDebug(1202) << "KonqView::slotCompleted hasPending=" << hasPending << endl;
+  //kdDebug(1202) << "KonqView::slotCompleted hasPending=" << hasPending << endl;
   m_pKonqFrame->statusbar()->slotLoadingProgress( -1 );
 
   if ( ! m_bLockHistory )
@@ -599,15 +615,21 @@ void KonqView::slotMouseOverInfo( const KFileItem *item )
   QApplication::sendEvent( m_pMainWindow, &ev );
 }
 
+void KonqView::setLocationBarURL( const KURL& locationBarURL )
+{
+  setLocationBarURL( locationBarURL.pathOrURL() );
+}
+
 void KonqView::setLocationBarURL( const QString & locationBarURL )
 {
-  kdDebug(1202) << "KonqView::setLocationBarURL " << locationBarURL << " this=" << this << endl;
+  //kdDebug(1202) << "KonqView::setLocationBarURL " << locationBarURL << " this=" << this << endl;
 
   m_sLocationBarURL = locationBarURL;
   if ( m_pMainWindow->currentView() == this )
   {
     //kdDebug(1202) << "is current view " << this << endl;
     m_pMainWindow->setLocationBarURL( m_sLocationBarURL );
+    m_pMainWindow->setPageSecurity( m_pageSecurity );
   }
   if (!m_bPassiveMode) setTabIcon( m_sLocationBarURL );
 }
@@ -618,6 +640,14 @@ void KonqView::setIconURL( const KURL & iconURL )
   m_bGotIconURL = true;
 }
 
+void KonqView::setPageSecurity( int pageSecurity )
+{
+  m_pageSecurity = (KonqMainWindow::PageSecurity)pageSecurity;
+
+  if ( m_pMainWindow->currentView() == this )
+    m_pMainWindow->setPageSecurity( m_pageSecurity );
+}
+
 void KonqView::setTabIcon( const  QString &url )
 {
   if (!m_bPassiveMode) frame()->setTabIcon( url, 0L );
@@ -702,7 +732,7 @@ void KonqView::updateHistoryEntry( bool saveLocationBarURL )
   }
 
 #ifdef DEBUG_HISTORY
-  kdDebug(1202) << "Saving part URL : " << m_pPart->url().url() << " in history position " << m_lstHistory.at() << endl;
+  kdDebug(1202) << "Saving part URL : " << m_pPart->url() << " in history position " << m_lstHistory.at() << endl;
 #endif
   current->url = m_pPart->url();
 
@@ -771,7 +801,12 @@ void KonqView::go( int steps )
   kdDebug(1202) << "New position " << m_lstHistory.at() << endl;
 #endif
 
-  HistoryEntry h( *currentHistoryEntry ); // make a copy of the current history entry, as the data
+  restoreHistory();
+}
+
+void KonqView::restoreHistory()
+{
+  HistoryEntry h( *(m_lstHistory.current()) ); // make a copy of the current history entry, as the data
                                           // the pointer points to will change with the following calls
 
 #ifdef DEBUG_HISTORY
@@ -815,6 +850,16 @@ void KonqView::go( int steps )
 #endif
 }
 
+const HistoryEntry * KonqView::historyAt(const int pos)
+{
+    if(pos<0 || pos>=(int)m_lstHistory.count())
+	return 0L;
+    int oldpos = m_lstHistory.at();     
+    const HistoryEntry* h = m_lstHistory.at(pos);
+    m_lstHistory.at( oldpos );
+    return h;
+}
+
 void KonqView::copyHistory( KonqView *other )
 {
     m_lstHistory.clear();
@@ -822,6 +867,7 @@ void KonqView::copyHistory( KonqView *other )
     QPtrListIterator<HistoryEntry> it( other->m_lstHistory );
     for (; it.current(); ++it )
         m_lstHistory.append( new HistoryEntry( *it.current() ) );
+    m_lstHistory.at(other->m_lstHistory.at());
 }
 
 KURL KonqView::url() const
@@ -860,9 +906,10 @@ void KonqView::setRun( KonqRun * run )
 
 void KonqView::stop()
 {
-  kdDebug(1202) << "KonqView::stop()" << endl;
+  //kdDebug(1202) << "KonqView::stop()" << endl;
   m_bAborted = false;
-  if ( m_bLoading || m_bPendingRedirection)
+  finishedWithCurrentURL();
+  if ( m_bLoading || m_bPendingRedirection )
   {
     // aborted -> confirm the pending url. We might as well remove it, but
     // we decided to keep it :)
@@ -889,6 +936,16 @@ void KonqView::stop()
     updateHistoryEntry(true);
 }
 
+void KonqView::finishedWithCurrentURL()
+{
+  if ( !m_tempFile.isEmpty() )
+  {
+    kdDebug(1202) << "######### Deleting tempfile after use:" << m_tempFile << endl;
+    QFile::remove( m_tempFile );
+    m_tempFile = QString::null;
+  }
+}
+
 void KonqView::setPassiveMode( bool mode )
 {
   // In theory, if m_bPassiveMode is true and mode is false,
