@@ -5,8 +5,6 @@
 			Matthias Ettrich
 			Matthias Elter
 
-    $Id: kpager.cpp,v 1.60 2003/11/11 16:50:37 lunakl Exp $
-
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
@@ -58,6 +56,7 @@
 #include "desktop.h"
 #include <qpopupmenu.h>
 #include <kpopupmenu.h>
+#include <kiconloader.h>
 #include <assert.h>
 
 KPagerMainWindow::KPagerMainWindow(QWidget *parent, const char *name)
@@ -341,7 +340,7 @@ void KPager::showPopupMenu( WId wid, QPoint pos)
             m_mnu->insertItem(i18n("&To Desktop"), m_dmnu );
             m_mnu->insertSeparator();
 
-            m_mnu->insertItem(i18n("&Close"), CloseOp);
+            m_mnu->insertItem(SmallIcon("fileclose"),i18n("&Close"), CloseOp);
 
             m_mnu->insertSeparator();
 	    m_prefs_action->plug(m_mnu);
