@@ -1,8 +1,8 @@
 /*
     KSysGuard, the KDE System Guard
-	   
-    Copyright (c) 2001 Tobias Koenig <tokoe82@yahoo.de>
-    
+
+    Copyright (c) 2001 Tobias Koenig <tokoe@kde.org>
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of version 2 of the GNU General Public
     License as published by the Free Software Foundation.
@@ -14,25 +14,27 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
 */
 
-#ifndef _diskstat_h_
-#define _diskstat_h_
+#ifndef KSG_DISKSTAT_H
+#define KSG_DISKSTAT_H
 
-void initDiskStat(void);
-void exitDiskStat(void);
+void initDiskStat( struct SensorModul* );
+void exitDiskStat( void );
 
-int updateDiskStat(void);
+int updateDiskStat( void );
+void checkDiskStat( void );
 
-void printDiskStat(const char* cmd);
-void printDiskStatInfo(const char* cmd);
+void printDiskStat( const char* );
+void printDiskStatInfo( const char* );
 
-void printDiskStatUsed(const char* cmd);
-void printDiskStatUsedInfo(const char* cmd);
-void printDiskStatFree(const char* cmd);
-void printDiskStatFreeInfo(const char* cmd);
-void printDiskStatPercent(const char* cmd);
-void printDiskStatPercentInfo(const char* cmd);
+void printDiskStatUsed( const char* );
+void printDiskStatUsedInfo( const char* );
+void printDiskStatFree( const char* );
+void printDiskStatFreeInfo( const char* );
+void printDiskStatPercent( const char* );
+void printDiskStatPercentInfo( const char* );
 
 #endif
