@@ -144,6 +144,7 @@ private:
     int bar_x_ofs;
     int in_unobs;
     QTime time;
+    bool resizable;
 };
 
 class B2ClientFactory : public QObject, public KDecorationFactory
@@ -153,6 +154,7 @@ public:
     virtual ~B2ClientFactory();
     virtual KDecoration *createDecoration(KDecorationBridge *);
     virtual bool reset(unsigned long changed);
+    virtual bool supports( Ability ability );
     QValueList< B2ClientFactory::BorderSize > borderSizes() const;
 };
 
