@@ -476,14 +476,23 @@ ClientList Workspace::constrainedStackingOrder()
     kdDebug() << "stacking1:" << endl;
 #endif
     // build the order from layers
+    QMap< Group*, Layer > minimum_layer;
     for( ClientList::ConstIterator it = unconstrained_stacking_order.begin();
          it != unconstrained_stacking_order.end();
          ++it )
         {
-#if 0
-        kdDebug() << (void*)(*it) << *it << endl;
-#endif
-        layer[ (*it)->layer() ].append( *it );
+        Layer l = (*it)->layer();
+        // If a window is raised above some other window in the same window group
+        // which is in a higher layer, make sure it stays above that window (see #95731).
+        if(( l == NormalLayer || l == AboveLayer )
+            && !(*it)->isDock()
+            && minimum_layer.contains( (*it)->group())
+            && l < minimum_layer[ (*it)->group() ] )
+            {
+            l = minimum_layer[ (*it)->group() ];
+            }
+        minimum_layer[ (*it)->group() ] = l;
+        layer[ l ].append( *it );
         }
     ClientList stacking;    
     for( Layer lay = FirstLayer;
@@ -616,13 +625,9 @@ bool Workspace::keepTransientAbove( const Client* mainwindow, const Client* tran
     // needs to be found.
     if( transient->isDialog() && !transient->isModal() && transient->groupTransient())
         return false;
-    return true;
     // #63223 - don't keep transients above docks, because the dock is kept high,
     // and e.g. dialogs for them would be too high too
-    // TODO this doesn't really work - the transient should be raised after clicking
-    // on the dock, but docks don't become active after clicking them
-    if( mainwindow->isDock() && !mainwindow->keepBelow()
-        && !mainwindow->isActive() && !transient->isActive()) // TODO !w->group()->isActive() ???
+    if( mainwindow->isDock())
         return false;
     return true;
     }
@@ -654,10 +659,9 @@ void Client::restackWindow( Window /*above TODO */, int detail, NET::RequestSour
 void Client::setKeepAbove( bool b )
     {
     b = rules()->checkKeepAbove( b );
-    if( b )
+    if( b && !rules()->checkKeepBelow( false ))
         setKeepBelow( false );
-    if ( b == keepAbove()
-        || ( b && keepBelow())) // forced below
+    if ( b == keepAbove())
         { // force hint change if different
         if( bool( info->state() & NET::KeepAbove ) != keepAbove())
             info->setState( keepAbove() ? NET::KeepAbove : 0, NET::KeepAbove );
@@ -674,10 +678,9 @@ void Client::setKeepAbove( bool b )
 void Client::setKeepBelow( bool b )
     {
     b = rules()->checkKeepBelow( b );
-    if( b )
+    if( b && !rules()->checkKeepAbove( false ))
         setKeepAbove( false );
-    if ( b == keepBelow()
-        || ( b && keepAbove())) // forced above
+    if ( b == keepBelow())
         { // force hint change if different
         if( bool( info->state() & NET::KeepBelow ) != keepBelow())
             info->setState( keepBelow() ? NET::KeepBelow : 0, NET::KeepBelow );
