@@ -30,6 +30,7 @@ License. See the file "COPYING" for the exact licensing terms.
 #include <kglobalaccel.h>
 #include <kkeynative.h>
 #include <kglobalsettings.h>
+#include <kiconeffect.h>
 #include <X11/keysym.h>
 #include <X11/keysymdef.h>
 
@@ -211,7 +212,7 @@ void TabBox::reset()
     // height, width for the popup
     h += 2 * frameWidth();
     w = 2*frameWidth() + 5*2 + ( showMiniIcon ? 16 : 32 ) + 8 + wmax; // 5*2=margins, ()=icon, 8=space between icon+text
-    w = kClamp( w, r.width()/3 , r.width() );
+    w = kClamp( w, r.width()/3 , r.width() * 4 / 5 );
 
     setGeometry( (r.width()-w)/2 + r.x(),
                  (r.height()-h)/2+ r.y(),
@@ -358,16 +359,24 @@ void TabBox::drawContents( QPainter * )
                     p.fillRect(x, y, r.width(), lineHeight, colorGroup().highlight());
 
                   // draw icon
+                  QPixmap icon;
                   if ( showMiniIcon )
                     {
                     if ( !(*it)->miniIcon().isNull() )
-                      p.drawPixmap( x+5, y + (lineHeight - iconWidth)/2, (*it)->miniIcon() );
+                      icon = (*it)->miniIcon();
                     }
                   else
                     if ( !(*it)->icon().isNull() )
-                      p.drawPixmap( x+5, y + (lineHeight - iconWidth)/2, (*it)->icon() );
+                      icon = (*it)->icon();
                     else if ( menu_pix )
-                      p.drawPixmap( x, y + (lineHeight - iconWidth)/2, *menu_pix );
+                      icon = *menu_pix;
+                
+                  if( !icon.isNull())
+                    {
+                    if( (*it)->isMinimized())
+                        KIconEffect::semiTransparent( icon );
+                    p.drawPixmap( x+5, y + (lineHeight - iconWidth)/2, icon );
+                    }
 
                   // generate text to display
                   QString s;
@@ -385,6 +394,23 @@ void TabBox::drawContents( QPainter * )
                   // draw text
                   if ( (*it) == currentClient() )
                     p.setPen(colorGroup().highlightedText());
+                  else if( (*it)->isMinimized())
+                    {
+                    QColor c1 = colorGroup().text();
+                    QColor c2 = colorGroup().background();
+                    // from kicker's TaskContainer::blendColors()
+                    int r1, g1, b1;
+                    int r2, g2, b2;
+
+                    c1.rgb( &r1, &g1, &b1 );
+                    c2.rgb( &r2, &g2, &b2 );
+
+                    r1 += (int) ( .5 * ( r2 - r1 ) );
+                    g1 += (int) ( .5 * ( g2 - g1 ) );
+                    b1 += (int) ( .5 * ( b2 - b1 ) );
+
+                    p.setPen(QColor( r1, g1, b1 ));
+                    }
                   else
                     p.setPen(colorGroup().text());
 
@@ -818,6 +844,7 @@ bool Workspace::startKDEWalkThroughWindows()
         return false;
     tab_grab        = TRUE;
     keys->setEnabled( false );
+    client_keys->setEnabled( false );
     tab_box->setMode( TabBox::WindowsMode );
     tab_box->reset();
     return TRUE;
@@ -829,6 +856,7 @@ bool Workspace::startWalkThroughDesktops( int mode )
         return false;
     control_grab = TRUE;
     keys->setEnabled( false );
+    client_keys->setEnabled( false );
     tab_box->setMode( (TabBox::Mode) mode );
     tab_box->reset();
     return TRUE;
@@ -980,6 +1008,7 @@ void Workspace::closeTabBox()
     removeTabBoxGrab();
     tab_box->hide();
     keys->setEnabled( true );
+    client_keys->setEnabled( true );
     tab_grab = FALSE;
     control_grab = FALSE;
     }
@@ -1027,6 +1056,7 @@ void Workspace::tabBoxKeyRelease( const XKeyEvent& ev )
         removeTabBoxGrab();
         tab_box->hide();
         keys->setEnabled( true );
+        client_keys->setEnabled( true );
         tab_grab = false;
         if( Client* c = tab_box->currentClient())
             {
@@ -1040,6 +1070,7 @@ void Workspace::tabBoxKeyRelease( const XKeyEvent& ev )
         removeTabBoxGrab();
         tab_box->hide();
         keys->setEnabled( true );
+        client_keys->setEnabled( true );
         control_grab = False;
         if ( tab_box->currentDesktop() != -1 )
             {
