@@ -23,6 +23,7 @@ License. See the file "COPYING" for the exact licensing terms.
 
 #include <kxerrorhandler.h>
 #include <assert.h>
+#include <kdebug.h>
 
 #include <X11/Xlib.h>
 #include <X11/extensions/shape.h>
@@ -164,7 +165,7 @@ bool KWinSelectionOwner::genericReply( Atom target_P, Atom property_P, Window re
     {
     if( target_P == xa_version )
         {
-        Q_INT32 version[] = { 2, 0 };
+        long version[] = { 2, 0 };
         XChangeProperty( qt_xdisplay(), requestor_P, property_P, XA_INTEGER, 32,
             PropModeReplace, reinterpret_cast< unsigned char* >( &version ), 2 );
         }
@@ -304,6 +305,50 @@ bool isLocalMachine( const QCString& host )
     return false;
     }
 
+#ifndef KCMRULES
+ShortcutDialog::ShortcutDialog( const KShortcut& cut )
+    : KShortcutDialog( cut, false /*TODO???*/ )
+    {
+    // make it a popup, so that it has the grab
+    XSetWindowAttributes attrs;
+    attrs.override_redirect = True;
+    XChangeWindowAttributes( qt_xdisplay(), winId(), CWOverrideRedirect, &attrs );
+    setWFlags( WType_Popup );
+    }
+
+void ShortcutDialog::accept()
+    {
+    for( int i = 0;
+         ;
+         ++i )
+        {
+        KKeySequence seq = shortcut().seq( i );
+        if( seq.isNull())
+            break;
+        if( seq.key( 0 ) == Key_Escape )
+            {
+            reject();
+            return;
+            }
+        if( seq.key( 0 ) == Key_Space )
+            { // clear
+            setShortcut( KShortcut());
+            KShortcutDialog::accept();
+            return;
+            }
+        if( seq.key( 0 ).modFlags() == 0 )
+            { // no shortcuts without modifiers
+            KShortcut cut = shortcut();
+            cut.setSeq( i, KKeySequence());
+            setShortcut( cut );
+            return;
+            }
+        }
+    KShortcutDialog::accept();
+    }
+#endif
+
+
 } // namespace
 
 #ifndef KCMRULES
