@@ -14,7 +14,7 @@ public:
     ~XKBExtension();
     bool init();
 
-    bool setXkbOptions(const QString& options, bool resetOldOptions);
+    static bool setXkbOptions(const QString& options, bool resetOldOptions);
     bool setLayout(const QString& model, const QString& layout, const char* variant, unsigned int group,
 				const char* baseGr);
     bool setGroup(unsigned int group);
